@@ -27,13 +27,12 @@
  * playing 8 or 16 bit data, immediate stop and synchronization. This library
  * currently provides OSS, NAS, ALSA and PulseAudio backend. The available backends are
  * specified at compile-time using the directives WITH_OSS, WITH_NAS, WITH_ALSA, 
- * WITH_PULSE, but the user program is allowed to switch between them at run-time.
+ * WITH_PULSE, WITH_LIBAO but the user program is allowed to switch between them at run-time.
  */
 
 #include "spd_audio.h"
 
 #include <stdio.h>
-#include <sys/soundcard.h>
 #include <sys/fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
@@ -47,6 +46,9 @@
 #ifdef WITH_OSS
 #include "oss.c"
 #endif
+#ifdef WITH_LIBAO
+#include "libao.c"
+#endif
 
 /* The NAS backend */
 #ifdef WITH_NAS
@@ -98,7 +100,7 @@ spd_audio_open(AudioOutputType type, void **pars, char **error)
 
     if (type == AUDIO_OSS){
 #ifdef WITH_OSS
-	id->function = (Funct*) &oss_functions;
+	id->function = &oss_functions;
 
 	if (id->function->open != NULL){
 	    ret = id->function->open(id, pars);
@@ -116,10 +118,11 @@ spd_audio_open(AudioOutputType type, void **pars, char **error)
 	*error = strdup("The sound library wasn't compiled with OSS support.");
 	return NULL;
 #endif       
+
     }
     else if (type == AUDIO_ALSA){
 #ifdef WITH_ALSA
-	id->function = (Funct*) &alsa_functions;
+	id->function = &alsa_functions;
 
 	if (id->function->open != NULL){
 	    ret = id->function->open(id, pars);
@@ -140,7 +143,7 @@ spd_audio_open(AudioOutputType type, void **pars, char **error)
     }
     else if (type == AUDIO_NAS){
 #ifdef WITH_NAS
-	id->function = (Funct*) &nas_functions;
+	id->function = &nas_functions;
 
 	if (id->function->open != NULL){
 	    ret = id->function->open(id, pars);
@@ -161,7 +164,7 @@ spd_audio_open(AudioOutputType type, void **pars, char **error)
     }
     else if (type == AUDIO_PULSE){
 #ifdef WITH_PULSE
-	id->function = (Funct*) &pulse_functions;
+	id->function = &pulse_functions;
 
 	if (id->function->open != NULL){
 	    ret = id->function->open(id, pars);
@@ -178,6 +181,27 @@ spd_audio_open(AudioOutputType type, void **pars, char **error)
 #else
 	*error = strdup("The sound library wasn't compiled with PulseAudio support.");
 	return NULL;
+#endif
+    }
+    else if (type == AUDIO_LIBAO){
+#ifdef WITH_LIBAO
+	id->function = &libao_functions;
+
+	if (id->function->open != NULL){
+	    ret = id->function->open(id, pars);
+	    if (ret != 0){
+		*error = (char*) strdup("Couldn't open libao");
+		return NULL;
+	    }
+	}
+	else{
+	    *error = (char*) strdup("Couldn't open libao  module.");
+	    return NULL;
+	}
+	id->type = AUDIO_LIBAO;
+#else
+	*error = strdup("The sound library wasn't compiled with libao support.");
+	return NULL;
 #endif
     }
     else{
@@ -291,7 +315,7 @@ Comments:
 int
 spd_audio_close(AudioID *id)
 {
-    int ret;
+    int ret = 0;
     if (id && id->function->close){
 	ret = id->function->close(id);
     }
@@ -336,3 +360,11 @@ spd_audio_set_volume(AudioID *id, int volume)
     id->volume = volume;
     return 0;
 }
+
+void
+spd_audio_set_loglevel(int level)
+{
+    if (level){
+        log_level = level;
+    }
+}
