@@ -1,7 +1,7 @@
 /*
   libspeechd.c - Shared library for easy acces to Speech Dispatcher functions
  *
- * Copyright (C) 2001, 2002, 2003, 2006, 2007 Brailcom, o.p.s.
+ * Copyright (C) 2001, 2002, 2003, 2006, 2007, 2008 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: libspeechd.c,v 1.36 2008-07-30 09:46:56 hanke Exp $
+ * $Id: libspeechd.c,v 1.37 2008-12-23 09:15:32 pdm Exp $
  */
 
 
@@ -27,6 +27,7 @@
 #include <sys/socket.h>
 #include <netinet/tcp.h>
 #include <netinet/in.h>
+#include <sys/un.h>
 #include <arpa/inet.h>
 #include <unistd.h>
 
@@ -48,6 +49,10 @@
 
 /* --------------  Private functions headers ------------------------*/
 
+#ifdef LIBSPEECHD_DEBUG
+FILE *spd_debug;
+#endif
+
 typedef struct {
     char *str;
     size_t bytes;
@@ -70,6 +75,73 @@ static _SPDString spd_string_new(void);
 
 pthread_mutex_t spd_logging_mutex;
 
+#if !(defined(__GLIBC__) || defined(_GNU_SOURCE))
+/* Added by Willie Walker - strndup and getline are gcc-isms */
+char *strndup ( const char *s, size_t n)
+{
+        size_t nAvail;
+        char *p;
+
+        if ( !s )
+                return 0;
+
+        if ( strlen(s) > n )
+                nAvail = n + 1;
+        else
+                nAvail = strlen(s) + 1;
+        p = malloc ( nAvail );
+        memcpy ( p, s, nAvail );
+        p[nAvail - 1] = '\0';
+
+        return p;
+}
+
+#define BUFFER_LEN 256
+ssize_t getline (char **lineptr, size_t *n, FILE *f)
+{
+        char ch;
+        size_t m = 0;
+        ssize_t buf_len = 0;
+        char * buf = NULL;
+        char * p = NULL;
+
+	if (errno != 0) {
+                SPD_DBG("getline: errno came in as %d!!!\n", errno);
+	        errno = 0;
+	}
+        while ( (ch = getc(f)) !=EOF )
+        {
+                if (errno != 0)
+                        return -1;
+                if ( m++ >= buf_len )
+                {
+                        buf_len += BUFFER_LEN;
+                        buf = (char *) realloc(buf, buf_len + 1);
+                        if ( buf == NULL )
+                        {
+                                SPD_DBG("buf==NULL");
+                                return -1;
+                        }
+                        p = buf + buf_len - BUFFER_LEN;
+                }
+                *p = ch;
+                p++;
+                if ( ch == '\n' )
+                        break;
+        }
+        if ( m == 0 )
+        {
+                SPD_DBG("getline: m=%d!",m);
+                return -1;
+        } else {
+                *p = '\0';
+                *lineptr = buf;
+                *n = m;
+                return m;
+        }
+}
+#endif /* !(defined(__GLIBC__) && defined(_GNU_SOURCE)) */
+
 /* --------------------- Public functions ------------------------- */
 
 #define SPD_REPLY_BUF_SIZE 65536
@@ -79,43 +151,47 @@ pthread_mutex_t spd_logging_mutex;
  * or -1 if no connection was opened. */
 
 SPDConnection*
-spd_open(const char* client_name, const char* connection_name, const char* user_name, SPDConnectionMode mode)
+spd_open(const char* client_name, const char* connection_name, const char* user_name,
+	 SPDConnectionMode mode)
+{
+    return spd_open2(client_name, connection_name, user_name,
+		     mode, SPD_METHOD_UNIX_SOCKET, 1);
+}
+
+SPDConnection*
+spd_open2(const char* client_name, const char* connection_name, const char* user_name,
+	  SPDConnectionMode mode, SPDConnectionMethod method, int autospawn)
 {
-    struct sockaddr_in address;
     SPDConnection *connection;
     char *set_client_name;
     char* conn_name;
     char* usr_name;
     char *env_port;
+    char *env_socket;
     int port;
     int ret;
     char tcp_no_delay = 1;
-    
+
+    /* Autospawn related */
+    const char *pidof_speechd[] = { "pidof", "speech-dispatcher", NULL };
+    const char *speechd_cmd[] = { SPD_SPAWN_CMD, "--spawn", NULL };
+    gchar *speechd_pid = NULL;
+    GError *error = NULL;
+    gint exit_status;
+
     if (client_name == NULL) return NULL;
     
     if (user_name == NULL)
-	usr_name = (char*) g_get_user_name();
+    {
+        usr_name = strdup((char*) g_get_user_name());
+    }
     else
-	usr_name = strdup(user_name);
+        usr_name = strdup(user_name);
     
     if(connection_name == NULL)
-	conn_name = strdup("main");
+        conn_name = strdup("main");
     else
-	conn_name = strdup(connection_name);
-    
-    env_port = getenv("SPEECHD_PORT");
-    if (env_port != NULL)
-	port = strtol(env_port, NULL, 10);
-    else
-	port = SPEECHD_DEFAULT_PORT;
-
-    connection = xmalloc(sizeof(SPDConnection));
-    
-    /* Prepare a new socket */
-    address.sin_addr.s_addr = inet_addr("127.0.0.1");
-    address.sin_port = htons(port);
-    address.sin_family = AF_INET;
-    connection->socket = socket(AF_INET, SOCK_STREAM, 0);
+        conn_name = strdup(connection_name);
     
 #ifdef LIBSPEECHD_DEBUG
     spd_debug = fopen("/tmp/libspeechd.log", "w");
@@ -130,13 +206,67 @@ spd_open(const char* client_name, const char* connection_name, const char* user_
     SPD_DBG("Debugging started");
 #endif /* LIBSPEECHD_DEBUG */
     
-  /* Connect to server */
-    ret = connect(connection->socket, (struct sockaddr *)&address, sizeof(address));
-    if (ret == -1){
-        SPD_DBG("Error: Can't connect to server: %s", strerror(errno));
-	return NULL;
+    /* Autospawn -- check if Dispatcher is not running and if so, start it*/
+    if (autospawn){
+      if (g_spawn_sync(NULL, (gchar**)pidof_speechd, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &speechd_pid, NULL,
+		       &exit_status, &error) && strlen(speechd_pid) == 0){
+	g_spawn_sync(NULL, (gchar**)speechd_cmd, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
+		     NULL, NULL, NULL, NULL, &exit_status, &error);
+      }
     }
 
+    /* Connect to server using the selected method */
+    connection = xmalloc(sizeof(SPDConnection));
+    if (method==SPD_METHOD_INET_SOCKET){    
+      struct sockaddr_in address_inet;
+
+      env_port = getenv("SPEECHD_PORT");
+      if (env_port != NULL)
+        port = strtol(env_port, NULL, 10);
+      else
+        port = SPEECHD_DEFAULT_PORT;
+    
+      address_inet.sin_addr.s_addr = inet_addr("127.0.0.1");
+      address_inet.sin_port = htons(port);
+      address_inet.sin_family = AF_INET;
+      connection->socket = socket(AF_INET, SOCK_STREAM, 0);
+      ret = connect(connection->socket, (struct sockaddr *)&address_inet, sizeof(address_inet));
+      if (ret == -1){
+        SPD_DBG("Error: Can't connect to localhost on port %d using inet sockets: %s", port, strerror(errno));
+        close(connection->socket);
+	return NULL;
+      }
+      setsockopt(connection->socket, IPPROTO_TCP, TCP_NODELAY, &tcp_no_delay, sizeof(int));
+
+    }else if (method==SPD_METHOD_UNIX_SOCKET){
+      struct sockaddr_un address_unix;
+      GString* socket_filename;
+      /* Determine address for the unix socket */
+      env_socket = getenv("SPEECHD_SOCKET");
+      if (env_socket){
+	socket_filename = g_string_new(env_socket);
+      }else{
+	const char *homedir = g_getenv("HOME");
+	if (!homedir)
+	  homedir = g_get_home_dir();
+	socket_filename = g_string_new("");
+	g_string_printf(socket_filename, "%s/.speech-dispatcher/speechd.sock", homedir);
+      }
+      /* Create the unix socket */
+      address_unix.sun_family = AF_UNIX;
+      strncpy (address_unix.sun_path, socket_filename->str, sizeof (address_unix.sun_path));
+      address_unix.sun_path[sizeof (address_unix.sun_path) - 1] = '\0';
+      connection->socket = socket(AF_UNIX, SOCK_STREAM, 0);
+      ret = connect(connection->socket, (struct sockaddr *)&address_unix, SUN_LEN(&address_unix));
+      if (ret == -1){
+        SPD_DBG("Error: Can't connect to unix socket %s: %s", socket_filename->str, strerror(errno));
+        close(connection->socket);
+	return NULL;
+      }
+      /* Clean up */
+      g_string_free(socket_filename, 0);
+    }else SPD_FATAL("Unsupported connection method to spd_open");
+
     connection->callback_begin = NULL;
     connection->callback_end = NULL;
     connection->callback_im = NULL;
@@ -174,8 +304,6 @@ spd_open(const char* client_name, const char* connection_name, const char* user_
 	}
     }
 
-    setsockopt(connection->socket, IPPROTO_TCP, TCP_NODELAY, &tcp_no_delay, sizeof(int));
-
     /* By now, the connection is created and operational */
 
     set_client_name = g_strdup_printf("SET SELF CLIENT_NAME \"%s:%s:%s\"", usr_name,
@@ -222,68 +350,92 @@ spd_close(SPDConnection* connection)
     xfree(connection);
 }
 
-/* Say TEXT with priority PRIORITY.
- * Returns msg_uid on success, -1 otherwise. */                            
-int
-spd_say(SPDConnection *connection, SPDPriority priority, const char* text)
+/* Helper functions for spd_say. */
+static inline int
+spd_say_prepare(SPDConnection *connection, SPDPriority priority,
+	const char* text, char **escaped_text)
 {
-    static char command[16];
-    char *etext;
-    int ret;
-    char *pret;
-    char *reply;
-    int err;
-    int msg_id;
-
-    if (text == NULL) return -1;
-
-    pthread_mutex_lock(connection->ssip_mutex);
+    int ret = 0;
 
     SPD_DBG("Text to say is: %s", text);
 
-    /* Set priority */
-    SPD_DBG("Setting priority");
-    ret = spd_set_priority(connection, priority);
-    if(ret) RET(-1); 
-    
-    /* Check if there is no escape sequence in the text */
-    etext = escape_dot(text);
-    if (etext == NULL) etext = (char*) text;
-  
-    /* Start the data flow */
-    SPD_DBG("Sending SPEAK");
-    sprintf(command, "SPEAK");
-    ret = spd_execute_command_wo_mutex(connection, command);
-    if(ret){     
-        SPD_DBG("Error: Can't start data flow!");
-        RET(-1);
+    /* Insure that there is no escape sequence in the text */
+    *escaped_text = escape_dot(text);
+    /* Caller is now responsible for escaped_text. */
+    if (*escaped_text == NULL) {	/* Out of memory. */
+	SPD_DBG("spd_say could not allocate memory.");
+	ret = -1;
+    } else {
+	/* Set priority */
+	SPD_DBG("Setting priority");
+	ret = spd_set_priority(connection, priority);
+	if (!ret) {
+	    /* Start the data flow */
+	    SPD_DBG("Sending SPEAK");
+	    ret = spd_execute_command_wo_mutex(connection, "speak");
+	    if (ret) {
+		SPD_DBG("Error: Can't start data flow!");
+	    }
+	}
     }
-  
+
+    return ret;
+}
+
+static inline int
+spd_say_sending(SPDConnection *connection, const char* text)
+{
+    int msg_id = -1;
+    int err = 0;
+    char *reply = NULL;
+    char *pret = NULL;
+
     /* Send data */
     SPD_DBG("Sending data");
-    pret = spd_send_data_wo_mutex(connection, etext, SPD_NO_REPLY);
-    if(pret==NULL){
-        SPD_DBG("Can't send data wo mutex");
-        RET(-1); 
-    }
-    
-
-    /* Terminate data flow */
-    SPD_DBG("Terminating data flow");
-    ret = spd_execute_command_with_reply(connection, "\r\n.", &reply);
-    if(ret){
-        SPD_DBG("Can't terminate data flow");
-        RET(-1); 
+    pret = spd_send_data_wo_mutex(connection, text, SPD_NO_REPLY);
+    if (pret==NULL) {
+	SPD_DBG("Can't send data wo mutex");
+    } else {
+	/* Terminate data flow */
+	SPD_DBG("Terminating data flow");
+	err = spd_execute_command_with_reply(connection, "\r\n.", &reply);
+	if (err) {
+	    SPD_DBG("Can't terminate data flow");
+	} else {
+	    msg_id = get_param_int(reply, 1, &err);
+	    if (err < 0) {
+		SPD_DBG("Can't determine SSIP message unique ID parameter.");
+		msg_id = -1;
+	    }
+	}
     }
 
-    msg_id = get_param_int(reply, 1, &err);
-    if (err < 0){
-       SPD_DBG("Can't determine SSIP message unique ID parameter.");
-       msg_id = -1;
-    }
     xfree(reply);
+    xfree(pret);
+    return msg_id;
+}
 
-    pthread_mutex_unlock(connection->ssip_mutex);
+/* Say TEXT with priority PRIORITY.
+ * Returns msg_uid on success, -1 otherwise. */
+int
+spd_say(SPDConnection *connection, SPDPriority priority, const char* text)
+{
+    char *escaped_text = NULL;
+    int msg_id = -1;
+    int prepare_failed = 0;
+
+    if (text != NULL) {
+    pthread_mutex_lock(connection->ssip_mutex);
+
+	prepare_failed = spd_say_prepare(connection, priority, text, &escaped_text);
+	if (!prepare_failed)
+	    msg_id = spd_say_sending(connection, escaped_text);
+
+	xfree(escaped_text);
+	pthread_mutex_unlock(connection->ssip_mutex);
+    } else {
+	SPD_DBG("spd_say called with a NULL argument for <text>");
+    }
 
     SPD_DBG("Returning from spd_say");
     return msg_id;
@@ -654,18 +806,18 @@ spd_w_set_voice_type(SPDConnection *connection, SPDVoiceType type, const char *w
         return spd_w_set_ ## param (connection, val, who); \
     }
 
-SPD_SET_COMMAND_INT(voice_rate, RATE, ((val >= -100) && (val <= +100)) );
-SPD_SET_COMMAND_INT(voice_pitch, PITCH, ((val >= -100) && (val <= +100)) );
-SPD_SET_COMMAND_INT(volume, VOLUME, ((val >= -100) && (val <= +100)) );
+SPD_SET_COMMAND_INT(voice_rate, RATE, ((val >= -100) && (val <= +100)) )
+SPD_SET_COMMAND_INT(voice_pitch, PITCH, ((val >= -100) && (val <= +100)) )
+SPD_SET_COMMAND_INT(volume, VOLUME, ((val >= -100) && (val <= +100)) )
 
-SPD_SET_COMMAND_STR(language, LANGUAGE);
-SPD_SET_COMMAND_STR(output_module, OUTPUT_MODULE);
-SPD_SET_COMMAND_STR(synthesis_voice, SYNTHESIS_VOICE);
+SPD_SET_COMMAND_STR(language, LANGUAGE)
+SPD_SET_COMMAND_STR(output_module, OUTPUT_MODULE)
+SPD_SET_COMMAND_STR(synthesis_voice, SYNTHESIS_VOICE)
 
-SPD_SET_COMMAND_SPECIAL(punctuation, SPDPunctuation);
-SPD_SET_COMMAND_SPECIAL(capital_letters, SPDCapitalLetters);
-SPD_SET_COMMAND_SPECIAL(spelling, SPDSpelling);
-SPD_SET_COMMAND_SPECIAL(voice_type, SPDVoiceType);
+SPD_SET_COMMAND_SPECIAL(punctuation, SPDPunctuation)
+SPD_SET_COMMAND_SPECIAL(capital_letters, SPDCapitalLetters)
+SPD_SET_COMMAND_SPECIAL(spelling, SPDSpelling)
+SPD_SET_COMMAND_SPECIAL(voice_type, SPDVoiceType)
 
 
 #undef SPD_SET_COMMAND_INT
@@ -895,13 +1047,13 @@ spd_execute_command_with_reply(SPDConnection *connection, char* command, char **
 
     buf = g_strdup_printf("%s\r\n", command);
     *reply = spd_send_data_wo_mutex(connection, buf, SPD_WAIT_REPLY);
+    xfree(buf);
+    buf = NULL;
     if(*reply==NULL){
         SPD_DBG("Can't send data wo mutex in spd_execute_command_with_reply");
 	return -1;
     }
 
-    xfree(buf);
-    
     r = ret_ok(*reply);
 
     if (!r) return -1;
@@ -1028,6 +1180,7 @@ get_reply(SPDConnection *connection)
     size_t N = 0;
     int bytes;
     char *reply;
+    gboolean errors = FALSE;
 
     str = g_string_new("");
 
@@ -1040,15 +1193,25 @@ get_reply(SPDConnection *connection)
 	    if (connection->stream != NULL)
 	      fclose(connection->stream);
 	    connection->stream = NULL;
-	    return NULL;
+	    errors = TRUE;
+	} else {
+	    g_string_append(str, line);
 	}
-	g_string_append(str, line);
 	/* terminate if we reached the last line (without '-' after numcode) */
-    }while( !((strlen(line) < 4) || (line[3] == ' ')));
+    }while(!errors &&  !((strlen(line) < 4) || (line[3] == ' ')));
     
-    /* The resulting message received from the socket is stored in reply */
-    reply = str->str;
-    g_string_free(str, FALSE);
+    xfree(line);	/* getline allocates with malloc. */
+
+    if (errors) {
+	/* Free the GString and its character data, and return NULL. */
+	g_string_free(str, TRUE);
+	reply = NULL;
+    } else {
+	    /* The resulting message received from the socket is stored in reply */
+	    reply = str->str;
+	/* Free the GString, but not its character data. */
+	    g_string_free(str, FALSE);
+    }
 
     return reply;
 }
