@@ -33,8 +33,10 @@
 extern "C" {
 #endif
     
+#ifdef LIBSPEECHD_DEBUG
 /* Debugging */
 FILE* spd_debug;
+#endif
 
 /* Unless there is an fatal error, it doesn't print anything */
 #define SPD_FATAL(msg) { printf("Fatal error (libspeechd) [%s:%d]:"msg, __FILE__, __LINE__); fflush(stdout); exit(EXIT_FAILURE); }
@@ -117,6 +119,11 @@ typedef enum{
     SPD_MODE_THREADED = 1
 }SPDConnectionMode;
 
+typedef enum{
+    SPD_METHOD_UNIX_SOCKET = 0,
+    SPD_METHOD_INET_SOCKET = 1
+}SPDConnectionMethod;
+
 typedef void (*SPDCallback)(size_t msg_id, size_t client_id, SPDNotificationType state);
 typedef void (*SPDCallbackIM)(size_t msg_id, size_t client_id, SPDNotificationType state, char *index_mark);
 
@@ -152,6 +159,8 @@ typedef struct{
 
 /* Openning and closing Speech Dispatcher connection */
 SPDConnection* spd_open(const char* client_name, const char* connection_name, const char* user_name, SPDConnectionMode mode);
+SPDConnection* spd_open2(const char* client_name, const char* connection_name, const char* user_name, SPDConnectionMode mode,
+			 SPDConnectionMethod method, int autospawn);
 
 void spd_close(SPDConnection* connection);
 
@@ -190,8 +199,8 @@ int spd_set_voice_type_all(SPDConnection*, SPDVoiceType type);
 int spd_set_voice_type_uid(SPDConnection*, SPDVoiceType type, unsigned int uid);
 
 int spd_set_synthesis_voice(SPDConnection*, const char *voice_name);
-int spd_set_synthesis_voice_type_all(SPDConnection*, const char *voice_name);
-int spd_set_synthesis_voice_type_uid(SPDConnection*, const char *voice_name, unsigned int uid);
+int spd_set_synthesis_voice_all(SPDConnection*, const char *voice_name);
+int spd_set_synthesis_voice_uid(SPDConnection*, const char *voice_name, unsigned int uid);
 
 int spd_set_data_mode(SPDConnection *connection, SPDDataMode mode);
 
