@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */
 
 
@@ -45,6 +45,12 @@
 #include <sys/un.h>
 #include <unistd.h>
 
+#if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
+/* Added by Willie Walker - TEMP_FAILURE_RETRY, strndup, and getline
+ * are gcc-isms
+ */
+ssize_t getline (char **lineptr, size_t *n, FILE *f);
+#endif /* !(defined(__GLIBC__) || defined(_GNU_SOURCE)) */
 
 /* Utilities */
 
@@ -363,7 +369,7 @@ static void serve ()
   while (1)
     {
       struct sockaddr_un client_address;
-      size_t client_address_len = sizeof (client_address);
+      socklen_t client_address_len = sizeof (client_address);
       Stream *s = malloc (sizeof (Stream));
       if (s == NULL)
         system_error ("memory allocation");
