@@ -24,7 +24,6 @@
 #include <fdset.h>
 #include <errno.h>
 #include "module_utils.h"
-/* #include "spd_audio.c" */
 #include <sys/poll.h>
 #include <fcntl.h>
 #include <langinfo.h>
@@ -34,7 +33,7 @@
 #define MODULE_VERSION  "0.3"
 
 // #define DEBUG_MODULE 1
-DECLARE_DEBUG();
+DECLARE_DEBUG()
 
 /* Thread and process control */
 static int cicero_speaking = 0;
@@ -225,6 +224,15 @@ module_init(char **status_info)
   return 0;
 }
 
+
+int
+module_audio_init(char **status_info){
+  /* The following statement has no effect and so was commented out */
+  //status_info == NULL;
+  return 0;
+}
+
+
 VoiceDescription**
 module_list_voices(void)
 {
