@@ -22,7 +22,7 @@
  * @author Lukas Loehrer
  * Based on ibmtts.c.
  *
- * $Id: espeak.c,v 1.8 2008-02-08 10:01:09 hanke Exp $
+ * $Id: espeak.c,v 1.11 2008-10-15 17:04:36 hanke Exp $
  */
 
 /* < Includes*/
@@ -54,7 +54,7 @@
 #define MODULE_VERSION  "0.1"
 
 #define DEBUG_MODULE 1
-DECLARE_DEBUG();
+DECLARE_DEBUG()
 
 #define DBG_WARN(e, msg)						\
 	if (Debug && !(e)) {						\
@@ -132,14 +132,6 @@ static int playback_queue_size = 0; /* Number of audio frames currently in queue
 static pthread_mutex_t playback_queue_mutex;
 pthread_cond_t playback_queue_condition;
 
-/* > */
-/* < Audio. */
-
-static AudioID *espeak_audio_id = NULL;
-static AudioOutputType espeak_audio_output_method;
-static void *espeak_audio_pars[10];
-
-/* > */
 
 /* When a voice is set, this is the baseline pitch of the voice.
    SSIP PITCH commands then adjust relative to this. */
@@ -197,25 +189,14 @@ static void* _espeak_stop_or_pause(void*);
 /* > */
 /* < Module configuration options*/
 
-MOD_OPTION_1_INT(EspeakPitchRange);
-MOD_OPTION_1_STR(EspeakPunctuationList);
-MOD_OPTION_1_INT(EspeakCapitalPitchRise);
-
-/* Audio options */
-MOD_OPTION_1_STR(EspeakAudioOutputMethod);
-MOD_OPTION_1_STR(EspeakOSSDevice);
-MOD_OPTION_1_STR(EspeakNASServer);
-MOD_OPTION_1_STR(EspeakALSADevice);
-MOD_OPTION_1_STR(EspeakPulseServer);
-MOD_OPTION_1_INT(EspeakPulseMaxLength);
-MOD_OPTION_1_INT(EspeakPulseTargetLength);
-MOD_OPTION_1_INT(EspeakPulsePreBuffering);
-MOD_OPTION_1_INT(EspeakPulseMinRequest);
-
-MOD_OPTION_1_INT(EspeakAudioChunkSize);
-MOD_OPTION_1_INT(EspeakAudioQueueMaxSize);
-MOD_OPTION_1_STR(EspeakSoundIconFolder);
-MOD_OPTION_1_INT(EspeakSoundIconVolume);
+MOD_OPTION_1_INT(EspeakPitchRange)
+MOD_OPTION_1_STR(EspeakPunctuationList)
+MOD_OPTION_1_INT(EspeakCapitalPitchRise)
+ 
+MOD_OPTION_1_INT(EspeakAudioChunkSize)
+MOD_OPTION_1_INT(EspeakAudioQueueMaxSize)
+MOD_OPTION_1_STR(EspeakSoundIconFolder)
+MOD_OPTION_1_INT(EspeakSoundIconVolume)
 
 /* > */
 /* < Public functions */
@@ -228,16 +209,6 @@ module_load(void)
 	REGISTER_DEBUG();
 
 	/* Options */
-	MOD_OPTION_1_STR_REG(EspeakAudioOutputMethod, "alsa");
-	MOD_OPTION_1_STR_REG(EspeakOSSDevice, "/dev/dsp");
-	MOD_OPTION_1_STR_REG(EspeakNASServer, "tcp/localhost:5450");
-	MOD_OPTION_1_STR_REG(EspeakALSADevice, "default");
-	MOD_OPTION_1_STR_REG(EspeakPulseServer, "default");
-	MOD_OPTION_1_INT_REG(EspeakPulseMaxLength, 132300);
-	MOD_OPTION_1_INT_REG(EspeakPulseTargetLength, 4410);
-	MOD_OPTION_1_INT_REG(EspeakPulsePreBuffering, 2200);
-	MOD_OPTION_1_INT_REG(EspeakPulseMinRequest, 880);
-
 	MOD_OPTION_1_INT_REG(EspeakAudioChunkSize, 2000);
 	MOD_OPTION_1_INT_REG(EspeakAudioQueueMaxSize, 20*22050);
 	MOD_OPTION_1_STR_REG(EspeakSoundIconFolder, "/usr/share/sounds/sound-icons/");
@@ -250,6 +221,8 @@ module_load(void)
 		EspeakCapitalPitchRise = 0;
 	}
 
+	module_audio_id = NULL;
+	
 	return OK;
 }
 
@@ -264,7 +237,6 @@ int
 module_init(char **status_info)
 {
 	int ret;
-	char *error;
 	const char *espeak_version;
 	GString *info;
     
@@ -305,51 +277,6 @@ module_init(char **status_info)
 
 	espeak_voice_list = espeak_list_synthesis_voices();
 
-	/* > */
-	/* <Audio setup*/
-
-	DBG("Espeak: Opening audio.");
-	if (!strcmp(EspeakAudioOutputMethod, "oss")){
-		DBG("Espeak: Using OSS sound output.");
-		espeak_audio_pars[0] = (void *) strdup(EspeakOSSDevice);
-		espeak_audio_pars[1] = NULL;
-		espeak_audio_id = spd_audio_open(AUDIO_OSS, (void**) espeak_audio_pars, &error);
-		espeak_audio_output_method = AUDIO_OSS;
-	}
-	else if (!strcmp(EspeakAudioOutputMethod, "nas")){
-		DBG("Espeak: Using NAS sound output.");
-		espeak_audio_pars[0] = (void *) EspeakNASServer;
-		espeak_audio_pars[1] = NULL;
-		espeak_audio_id = spd_audio_open(AUDIO_NAS, (void**) espeak_audio_pars, &error);
-		espeak_audio_output_method = AUDIO_NAS;
-	}
-	else if (!strcmp(EspeakAudioOutputMethod, "alsa")){
-		DBG("Espeak: Using ALSA sound output.");
-		espeak_audio_pars[0] = (void *) EspeakALSADevice;
-		espeak_audio_pars[1] = NULL;
-		espeak_audio_id = spd_audio_open(AUDIO_ALSA, (void**) espeak_audio_pars, &error);
-		espeak_audio_output_method = AUDIO_ALSA;
-	} else if (!strcmp(EspeakAudioOutputMethod, "pulse")){
-		DBG("Espeak: Using PulseAudio sound output.");
-		espeak_audio_pars[0] = (void *) EspeakPulseServer;
-		espeak_audio_pars[1] = (void *) EspeakPulseMaxLength;
-		espeak_audio_pars[2] = (void *) EspeakPulseTargetLength;
-		espeak_audio_pars[3] = (void *) EspeakPulsePreBuffering;
-		espeak_audio_pars[4] = (void *) EspeakPulseMinRequest;
-		espeak_audio_pars[5] = NULL;
-		espeak_audio_id = spd_audio_open(AUDIO_PULSE, (void**) espeak_audio_pars, &error);
-		espeak_audio_output_method = AUDIO_PULSE;
-	} else{
-		ABORT("Sound output method specified in configuration not supported. "
-			  "Please choose 'oss', 'nas', or 'alsa'.");
-	}
-	if (espeak_audio_id == NULL){
-		g_string_append_printf(info, "Opening sound device failed. Reason: %s. ", error);
-		ABORT("Can't open sound device.");
-	}
-
-	/* > */
-
 	/* Reset global state */
 	espeak_state_reset();
 
@@ -380,14 +307,20 @@ module_init(char **status_info)
 		ABORT("Failed to create playback thread.");
 	}
 
-	/* > */
-    
 	*status_info = strdup("Espeak: Initialized successfully.");
 
 	return OK;
 }
 #undef ABORT
 
+
+int
+module_audio_init(char **status_info){
+  DBG("Opening audio");
+  return module_audio_init_spd(status_info);
+}
+
+
 VoiceDescription**
 module_list_voices(void)
 {
@@ -403,13 +336,17 @@ module_speak(gchar *data, size_t bytes, EMessageType msgtype)
     
 	DBG("Espeak: module_speak().");
 
+	pthread_mutex_lock(&espeak_state_mutex);
 	if (espeak_state != IDLE) {
 		DBG("Espeak: Warning, module_speak called when not ready.");
+		pthread_mutex_unlock(&espeak_state_mutex);
 		return FALSE;
 	}
 
-	if (0 != module_write_data_ok(data)) return FATAL_ERROR;
-
+	if (0 != module_write_data_ok(data)){
+	  pthread_mutex_unlock(&espeak_state_mutex);
+	  return FATAL_ERROR;
+	}
 	DBG("Espeak: Requested data: |%s| %d %ld", data, msgtype, bytes);
 
 	espeak_state_reset();
@@ -485,7 +422,9 @@ module_speak(gchar *data, size_t bytes, EMessageType msgtype)
 	if (result != EE_OK) {
 		return FALSE;
 	}
-    
+
+	pthread_mutex_unlock(&espeak_state_mutex);    
+
 	DBG("Espeak: Leaving module_speak() normally.");
 	return bytes;
 }
@@ -556,8 +495,8 @@ module_close(int status)
 	espeak_Terminate();
 
 	DBG("Espeak: Closing audio output");
-	if (espeak_audio_id) {
-		spd_audio_close(espeak_audio_id);
+	if (module_audio_id) {
+		spd_audio_close(module_audio_id);
 	}
 
 	DBG("Freeing resources.");
@@ -630,12 +569,12 @@ _espeak_stop_or_pause(void* nothing)
 		pthread_cond_broadcast(&playback_queue_condition);
 		pthread_mutex_unlock(&playback_queue_mutex);
 	
-		if (espeak_audio_id) {
+		if (module_audio_id) {
 			DBG("Espeak: Stopping audio.");
-			ret = spd_audio_stop(espeak_audio_id);
+			ret = spd_audio_stop(module_audio_id);
 			DBG_WARN(ret == 0, "spd_audio_stop returned non-zero value.");
 			while (is_thread_busy(&espeak_play_suspended_mutex))  {
-				ret = spd_audio_stop(espeak_audio_id);
+				ret = spd_audio_stop(module_audio_id);
 				DBG_WARN(ret == 0, "spd_audio_stop returned non-zero value.");
 				g_usleep(5000);
 			}
@@ -664,7 +603,7 @@ _espeak_stop_or_pause(void* nothing)
 		}
 	
 		DBG("Espeak: Stop or pause thread ended.......\n")
-			}
+		  }
 	pthread_exit(NULL);
 }
 
@@ -1085,8 +1024,8 @@ espeak_send_to_audio(TPlaybackQueueEntry *playback_queue_entry)
 
 	DBG("Espeak: Sending %i samples to audio.", track.num_samples);
 	/* Volume is controlled by the synthesizer.  Always play at normal on audio device. */
-	spd_audio_set_volume(espeak_audio_id, 85);
-	ret = spd_audio_play(espeak_audio_id, track, SPD_AUDIO_LE);
+	spd_audio_set_volume(module_audio_id, 85);
+	ret = spd_audio_play(module_audio_id, track, SPD_AUDIO_LE);
 	if (ret < 0) {
 		DBG("ERROR: Can't play track for unknown reason.");
 		return FALSE;
@@ -1207,8 +1146,8 @@ espeak_play_file(char *filename)
 	sf = sf_open(filename, SFM_READ, &sfinfo);
 	subformat = sfinfo.format & SF_FORMAT_SUBMASK ;
 	items = sfinfo.channels * sfinfo.frames;
-	DBG("Espeak: frames = %ld, channels = %d", (long) sfinfo.frames, sfinfo.channels);
-	DBG("Espeak: samplerate = %i, items = %Ld", sfinfo.samplerate, items);
+	DBG("Espeak: frames = %ld, channels = %d", sfinfo.frames, sfinfo.channels);
+	DBG("Espeak: samplerate = %i, items = %Ld", sfinfo.samplerate, (long long) items);
 	DBG("Espeak: major format = 0x%08X, subformat = 0x%08X, endian = 0x%08X",
 		sfinfo.format & SF_FORMAT_TYPEMASK, subformat, sfinfo.format & SF_FORMAT_ENDMASK);
 	if (sfinfo.channels < 1 || sfinfo.channels > 2) {
@@ -1237,14 +1176,14 @@ espeak_play_file(char *filename)
 		goto cleanup1;
 	}
 	readcount = sf_read_short(sf, (short *) track.samples, items);
-	DBG("Espeak: read %Ld items from audio file.", readcount);
+	DBG("Espeak: read %Ld items from audio file.", (long long) readcount);
 
 	if (readcount > 0) {
 		track.num_samples = readcount / sfinfo.channels;
 		DBG("Espeak: Sending %i samples to audio.", track.num_samples);
 		/* Volume is controlled by the synthesizer.  Always play at normal on audio device. */
-		spd_audio_set_volume(espeak_audio_id, EspeakSoundIconVolume);
-		int ret = spd_audio_play(espeak_audio_id, track, SPD_AUDIO_LE);
+		spd_audio_set_volume(module_audio_id, EspeakSoundIconVolume);
+		int ret = spd_audio_play(module_audio_id, track, SPD_AUDIO_LE);
 		if (ret < 0) {
 			DBG("ERROR: Can't play track for unknown reason.");
 			result = FALSE;
@@ -1283,7 +1222,7 @@ espeak_list_synthesis_voices()
 
 			voice->name = g_strdup(v->name);
 
-			gchar *first_lang = v->languages + 1;
+			const gchar *first_lang = v->languages + 1;
 			gchar *lang = NULL;
 			gchar *dialect = NULL;
 			if (g_utf8_validate(first_lang, -1, NULL)) {
