@@ -18,7 +18,7 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  *
- * $Id: module_utils.c,v 1.51 2008-02-08 10:01:09 hanke Exp $
+ * $Id: module_utils.c,v 1.55 2008-07-10 15:37:18 hanke Exp $
  */
 
 #include "fdsetconv.h"
@@ -26,8 +26,57 @@
 
 #include "module_utils.h"
 
+
 extern char* module_index_mark;
 
+#if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
+/* Added by Willie Walker - getline is a gcc-ism */
+#define BUFFER_LEN 256
+ssize_t getline (char **lineptr, size_t *n, FILE *f)
+{
+        char ch;
+        size_t m = 0;
+        ssize_t buf_len = 0;
+        char * buf = NULL;
+        char * p = NULL;
+
+	if (errno != 0) {
+                DBG("getline: errno came in as %d!!!\n", errno);
+	        errno = 0;
+	}
+        while ( (ch = getc(f)) !=EOF )
+        {
+                if (errno != 0)
+                        return -1;
+                if ( m++ >= buf_len )
+                {
+                        buf_len += BUFFER_LEN;
+                        buf = (char *) realloc(buf, buf_len + 1);
+                        if ( buf == NULL )
+                        {
+                                DBG("buf==NULL");
+                                return -1;
+                        }
+                        p = buf + buf_len - BUFFER_LEN;
+                }
+                *p = ch;
+                p++;
+                if ( ch == '\n' )
+                        break;
+        }
+        if ( m == 0 )
+        {
+                DBG("getline: m=%d!",m);
+                return -1;
+        } else {
+                *p = '\0';
+                *lineptr = buf;
+                *n = m;
+                return m;
+        }
+}
+#endif /* !(defined(__GLIBC__) || defined(_GNU_SOURCE)) */
+
 void*
 xmalloc(size_t size)
 {
@@ -227,6 +276,183 @@ do_set(void)
     return strdup("401 ERROR INTERNAL"); /* Can't be reached */
 }
 
+#define SET_AUDIO_NUM(name, cond) \
+ if(!strcmp(cur_item, #name)){ \
+     number = strtol(cur_value, &tptr, 10); \
+     if(!(cond)){ err = 2; continue; } \
+     if (tptr == cur_value){ err = 2; continue; } \
+     audio_settings.name = number; \
+ }
+
+#define SET_AUDIO_STR(name) \
+ if(!strcmp(cur_item, #name)){ \
+     xfree(audio_settings.name); \
+     if(!strcmp(cur_value, "NULL")) audio_settings.name = NULL; \
+     else audio_settings.name = strdup(cur_value); \
+ }
+
+
+char*
+do_audio(void)
+{
+    char *cur_item = NULL;
+    char *cur_value = NULL;
+    char *line = NULL;
+    int ret;
+    size_t n;
+    int number; char *tptr;
+    int err = 0;                /* Error status */
+    char *status;
+    char *msg;
+
+    printf("207 OK RECEIVING AUDIO SETTINGS\n");
+    fflush(stdout);
+
+    while(1){
+        line = NULL; n = 0;
+        ret = getline(&line, &n, stdin);
+        if (ret == -1){ err=1; break; }
+        if (!strcmp(line, ".\n")){
+            xfree(line);
+            break;
+        }
+        if (!err){
+            cur_item = strtok(line, "=");
+            if (cur_item == NULL){ err=1; continue; }
+            cur_value = strtok(NULL, "\n");
+            if (cur_value == NULL){ err=1; continue; }
+            
+            SET_AUDIO_STR(audio_output_method)
+            else SET_AUDIO_STR(audio_oss_device)
+	    else SET_AUDIO_STR(audio_alsa_device)
+	    else SET_AUDIO_STR(audio_nas_server)
+            else SET_AUDIO_STR(audio_pulse_server)
+	    else SET_AUDIO_NUM(audio_pulse_min_length, 1)
+            else err=2;             /* Unknown parameter */
+        }
+        xfree(line);
+    }
+
+    if (err == 1) return strdup("302 ERROR BAD SYNTAX");
+    if (err == 2) return strdup("303 ERROR INVALID PARAMETER OR VALUE");
+
+    err = module_audio_init(&status);
+
+    if (err == 0)
+      msg = g_strdup_printf("203 OK AUDIO INITIALIZED");
+    else
+      msg = g_strdup_printf("300-%s\n300 UNKNOWN ERROR", status);
+    
+    return msg;
+}
+
+#define SET_LOGLEVEL_NUM(name, cond) \
+ if(!strcmp(cur_item, #name)){ \
+     number = strtol(cur_value, &tptr, 10); \
+     if(!(cond)){ err = 2; continue; } \
+     if (tptr == cur_value){ err = 2; continue; } \
+     log_level = number; \
+     spd_audio_set_loglevel(log_level); \
+ }
+
+char*
+do_loglevel(void)
+{
+    char *cur_item = NULL;
+    char *cur_value = NULL;
+    char *line = NULL;
+    int ret;
+    size_t n;
+    int number; char *tptr;
+    int err = 0;                /* Error status */
+    char *status;
+    char *msg;
+
+    printf("207 OK RECEIVING LOGLEVEL SETTINGS\n");
+    fflush(stdout);
+
+    while(1){
+        line = NULL; n = 0;
+        ret = getline(&line, &n, stdin);
+        if (ret == -1){ err=1; break; }
+        if (!strcmp(line, ".\n")){
+            xfree(line);
+            break;
+        }
+        if (!err){
+            cur_item = strtok(line, "=");
+            if (cur_item == NULL){ err=1; continue; }
+            cur_value = strtok(NULL, "\n");
+            if (cur_value == NULL){ err=1; continue; }
+
+            SET_LOGLEVEL_NUM(log_level, 1)
+            else err=2;             /* Unknown parameter */
+        }
+        xfree(line);
+    }
+
+    if (err == 1) return strdup("302 ERROR BAD SYNTAX");
+    if (err == 2) return strdup("303 ERROR INVALID PARAMETER OR VALUE");
+
+    msg = g_strdup_printf("203 OK LOG LEVEL SET");
+
+    return msg;
+}
+
+char*
+do_debug(char* cmd_buf)
+{
+  /* TODO: Develop the full on/off logic etc. */
+
+  char **cmd;
+  char *filename;
+
+  cmd = g_strsplit (cmd_buf, " ", -1);
+
+  if (!cmd[1]){
+    g_strfreev(cmd);
+    return strdup("302 ERROR BAD SYNTAX");
+  }
+
+  if (!strcmp(cmd[1], "ON")){
+    if (!cmd[2]){
+      g_strfreev(cmd);
+      return strdup("302 ERROR BAD SYNTAX");
+    }
+
+
+    filename = cmd[2];
+    DBG("Additional logging into specific path %s requested", filename);
+    CustomDebugFile = fopen(filename, "w+");
+    if (CustomDebugFile == NULL){
+      DBG("ERROR: Can't open custom debug file for logging: %d (%s)",
+	  errno, strerror(errno));
+      return strdup("303 CANT OPEN CUSTOM DEBUG FILE");
+    }
+    if (Debug == 1)
+      Debug = 3;
+    else
+      Debug = 2;
+
+    DBG("Additional logging initialized");
+  }else if (!strcmp(cmd[1], "OFF")){
+    if (Debug == 3)
+      Debug = 1;
+    else 
+      Debug = 0;
+
+    if (CustomDebugFile != NULL)
+      fclose(CustomDebugFile);
+    CustomDebugFile = NULL;
+    DBG("Additional logging into specific path terminated");
+  }else{
+    return strdup("302 ERROR BAD SYNTAX");
+  }
+
+  g_strfreev(cmd);
+  return g_strdup("200 OK DEBUGGING ON");
+}
+
 char *
 do_list_voices(void)
 {
@@ -263,13 +489,13 @@ do_list_voices(void)
 /* This has to return int (although it doesn't return at all) so that we could
  * call it from PROCESS_CMD() macro like the other commands that return
  * something */
-int
+void
 do_quit(void)
 {
     printf("210 OK QUIT\n");    
     fflush(stdout);
     module_close(0);
-    return 0;
+    return;
 }
 
 int
@@ -305,7 +531,7 @@ module_get_message_part(const char* message, char* part, unsigned int *pos,
         if ((len-1-i) > 2){
             if ((message[*pos+1] == ' ') || (message[*pos+1] == '\n')
                 || (message[*pos+1] == '\r')){
-                for(n = 0; n <= num_dividers; n++){
+                for(n = 0; n <= num_dividers-1; n++){
                     if ((part[i] == dividers[n])){                        
                         part[i+1] = 0;                
                         (*pos)++;
@@ -879,3 +1105,100 @@ module_utils_init(void)
 
     return 0;
 }
+
+
+#define ABORT(msg) g_string_append(info, msg); \
+	*status_info = info->str; \
+	g_string_free(info, 0); \
+	return -1;
+
+int
+module_audio_init_spd(char **status_info)
+{
+  char * error;
+  GString *info;
+  char *module_audio_pars[10];
+  char *outputs;
+  int audio_output_set = 0;
+
+  info = g_string_new("");
+  outputs = audio_settings.audio_output_method;
+
+    DBG("Openning audio output system");
+    if (outputs[0]){
+	char *next = outputs;
+	size_t len;
+
+	do{
+	    outputs = next;
+	    next = strchr(outputs, ',');
+
+	    if (!outputs[0] || outputs[0] == ',')
+		continue;
+
+	    len = (next ? ((size_t)(next-outputs)) : strlen(outputs));
+
+	    if (len == 3 && strncmp("oss", outputs, len) == 0){
+		DBG("Using OSS audio output method");
+		module_audio_pars[0] = strdup(audio_settings.audio_oss_device);
+		module_audio_pars[1] = NULL;
+		module_audio_id = spd_audio_open(AUDIO_OSS, (void**) module_audio_pars, &error);
+		if (module_audio_id){
+		    module_audio_output_method = AUDIO_OSS;
+		    audio_output_set = 1;
+		}
+	    } else if (len == 5 && strncmp("libao", outputs, len) == 0){
+		DBG("Using libao audio output method");
+		module_audio_pars[0] = NULL;
+		module_audio_pars[1] = NULL;
+		module_audio_id = spd_audio_open(AUDIO_LIBAO, (void**) module_audio_pars, &error);
+		if (module_audio_id){
+		    module_audio_output_method = AUDIO_LIBAO;
+		    audio_output_set = 1;
+		}
+	    } else if (len == 4 && strncmp("alsa", outputs, len) == 0){
+		DBG("Using Alsa audio output method");
+		module_audio_pars[0] = audio_settings.audio_alsa_device;
+		module_audio_pars[1] = NULL;
+		module_audio_id = spd_audio_open(AUDIO_ALSA, (void**) module_audio_pars, &error);
+		if (module_audio_id){
+		    module_audio_output_method = AUDIO_ALSA;
+		    audio_output_set = 1;
+		    break;
+		}
+	    } else if (len == 3 && strncmp("nas", outputs, len) == 0){
+		DBG("Using NAS audio output method");
+		module_audio_pars[0] = audio_settings.audio_nas_server;
+		module_audio_pars[1] = NULL;
+		module_audio_id = spd_audio_open(AUDIO_NAS, (void**) module_audio_pars, &error);
+		if (module_audio_id){
+		    module_audio_output_method = AUDIO_NAS;
+		    audio_output_set = 1;
+		    break;
+		}
+	    } else if (len == 5 && strncmp("pulse", outputs, len) == 0){
+		DBG("Using PulseAudio output method");
+		module_audio_pars[0] = (void *) audio_settings.audio_pulse_server;
+		module_audio_pars[1] = (void *) audio_settings.audio_pulse_min_length;
+		module_audio_id = spd_audio_open(AUDIO_PULSE, (void**) module_audio_pars, &error);
+		if (module_audio_id){
+		    module_audio_output_method = AUDIO_PULSE;
+		    audio_output_set = 1;
+		    break;
+		}
+	    } else{
+		ABORT("Sound output method specified in configuration not supported. "
+		    "Please choose 'oss', 'alsa', 'nas' or 'pulse'.");
+	    }
+	} while(next++);
+    }
+
+    if (!audio_output_set){
+	g_string_append_printf(info, "Opening sound device failed. Reason: %s. ", error);
+	ABORT("Can't open sound device.");
+    }
+
+    return 0;
+}
+
+#undef ABORT
