@@ -25,13 +25,14 @@ A more convenient interface is provided by the 'Speaker' class.
 
 #TODO: Blocking variants for speak, char, key, sound_icon.
 
-import socket, sys, os
+import socket, sys, os, subprocess, time, tempfile
 
 try:
     import threading
 except:
     import dummy_threading as threading
 
+import paths
     
 class CallbackType(object):
     """Constants describing the available types of callbacks"""
@@ -94,7 +95,17 @@ class SSIPDataError(SSIPResponseError):
         return self._data
 
     
-class _SSIP_Connection:
+class SpawnError(Exception):
+    """Indicates failure in server autospawn."""
+
+class ConnectionMethod(object):
+    """Constants describing the possible methods of connection to server."""
+    UNIX_SOCKET = 'unix_socket'
+    """Unix socket communication using a filesystem path"""
+    INET_SOCKET = 'inet_socket'
+    """Inet socket communication using a host and port"""
+
+class _SSIP_Connection(object):
     """Implemantation of low level SSIP communication."""
     
     _NEWLINE = "\r\n"
@@ -110,13 +121,29 @@ class _SSIP_Connection:
                           704: CallbackType.PAUSE,
                           705: CallbackType.RESUME,
                           }
-    
-    def __init__(self, host, port):
+
+    def __init__(self, method, socket_path, host, port):
         """Init connection: open the socket to server,
         initialize buffers, launch a communication handling
-        thread."""
-        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self._socket.connect((socket.gethostbyname(host), port))
+        thread.
+        """
+
+        if method == ConnectionMethod.UNIX_SOCKET:
+            socket_family = socket.AF_UNIX
+            socket_connect_args = (socket_path,)
+        elif method == ConnectionMethod.INET_SOCKET:
+            assert host and port
+            socket_family = socket.AF_INET
+            socket_connect_args = (socket.gethostbyname(host), port)
+        else:
+            raise ValueError("Unsupported communication method")
+
+        try:
+            self._socket = socket.socket(socket_family, socket.SOCK_STREAM)
+            self._socket.connect(*socket_connect_args)
+        except socket.error:
+            raise SSIPCommunicationError("Can't open socket using method" + method)
+
         self._buffer = ""
         self._com_buffer = []
         self._callback = None
@@ -303,7 +330,6 @@ class _SSIP_Connection:
         """
         self._callback = callback
 
-            
 class Scope(object):
     """An enumeration of valid SSIP command scopes.
 
@@ -366,13 +392,16 @@ class SSIPClient(object):
 
     """
     
-    DEFAULT_SPEECHD_HOST = '127.0.0.1'
+    DEFAULT_HOST = '127.0.0.1'
     """Default host for server connections."""
-    DEFAULT_SPEECHD_PORT = 6560
+    DEFAULT_PORT = 6560
     """Default port number for server connections."""
+    DEFAULT_SOCKET_PATH = "~/.speech-dispatcher/speechd.sock"
+    """Default name of the communication unix socket"""
     
     def __init__(self, name, component='default', user='unknown', host=None,
-                 port=None):
+                 port=None, method=ConnectionMethod.UNIX_SOCKET, socket_path=None,
+                 autospawn=None):
         """Initialize the instance and connect to the server.
 
         Arguments:
@@ -381,25 +410,53 @@ class SSIPClient(object):
             multiple connections, this can be used to identify each of them.
           user -- user identification string (user name).  When multi-user
             acces is expected, this can be used to identify their connections.
-          host -- server hostname or IP address as a string.  If None, the
-            default value is taken from SPEECHD_HOST environment variable (if it
-            exists) or from the DEFAULT_SPEECHD_HOST attribute of this class.
-          port -- server port as number or None.  If None, the default value is
-            taken from SPEECHD_PORT environment variable (if it exists) or from the
-            DEFAULT_SPEECHD_PORT attribute of this class.
+          method -- communication method to use, one of the constants defined in class
+            ConnectionMethod
+          socket_path -- for ConnectionMethod.UNIX_SOCKET, socket
+            path in filesystem. By default, this is ~/.speech-dispatcher/speechd.sock
+            where `~' is the users home directory as determined from the system
+            defaults and from the $HOMEDIR environment variable.
+          host -- for ConnectionMethod.INET_SOCKET, server hostname
+            or IP address as a string.  If None, the default value is
+            taken from SPEECHD_HOST environment variable (if it
+            exists) or from the DEFAULT_HOST attribute of this class.
+          port -- for ConnectionMethod.INET_SOCKET method, server
+            port as number or None.  If None, the default value is
+            taken from SPEECHD_PORT environment variable (if it
+            exists) or from the DEFAULT_PORT attribute of this class.
+          autospawn -- a flag to specify whether the library should
+            try to start the server if it determines its not already
+            running or not
         
         For more information on client identification strings see Speech
         Dispatcher documentation.
           
         """
+        if socket_path is None:
+            socket_path = os.environ.get('SPEECHD_SOCKET',
+                                         os.path.expanduser(self.DEFAULT_SOCKET_PATH))
         if host is None:
-            host = os.environ.get('SPEECHD_HOST', self.DEFAULT_SPEECHD_HOST)
+            host = os.environ.get('SPEECHD_HOST', self.DEFAULT_HOST)
         if port is None:
             try:
                 port = int(os.environ.get('SPEECHD_PORT'))
             except (ValueError, TypeError):
-                port = self.DEFAULT_SPEECHD_PORT
-        self._conn = conn = _SSIP_Connection(host, port)
+                port = self.DEFAULT_PORT
+
+        connection_args = {'method': method, 'socket_path': socket_path,
+                           'host': host, 'port': port}
+        try:
+            self._conn = conn = _SSIP_Connection(**connection_args)
+        except SSIPCommunicationError:
+            # Suppose server might not be running, try the autospawn mechanism
+            if autospawn != False:
+                # Autospawn is however not guaranteed to start the server. The server
+                # will decide, based on it's configuration, whether to honor the request.
+                self._server_spawn()
+                self._conn = conn = _SSIP_Connection(**connection_args)
+            else:
+                raise
+
         full_name = '%s:%s:%s' % (user, name, component)
         conn.send_command('SET', Scope.SELF, 'CLIENT_NAME', full_name)
         code, msg, data = conn.send_command('HISTORY', 'GET', 'CLIENT_ID')
@@ -437,8 +494,21 @@ class SSIPClient(object):
                     del self._callbacks[msg_id]
         finally:
             self._lock.release()
-                
-        
+
+    def _server_spawn(self):
+        """Attempts to spawn the speech-dispatcher server."""
+        if os.path.exists(paths.SPD_SPAWN_CMD):
+            server = subprocess.Popen([paths.SPD_SPAWN_CMD, "--spawn"], stdin=None,
+                                      stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+            #TODO: When logging will be implemented in this library, it will be very
+            #desirable to read the stdout output and log it because it contains valuable
+            #information on the result of the autospawn and the reason for it
+            server.wait()
+            return server.pid
+        else:
+            raise SpawnError("Can't find Speech Dispatcher spawn command %s"
+                                         % (paths.SPD_SPAWN_CMD,))
+
     def set_priority(self, priority):
         """Set the priority category for the following messages.
 
