@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: config.c,v 1.16 2008-02-11 14:01:13 hanke Exp $
+ * $Id: config.c,v 1.18 2009-05-14 08:11:33 hanke Exp $
  */
 
 #include <dotconf.h>
@@ -33,7 +33,6 @@ static TFDSetClientSpecific *cl_spec_section;
 /* So that gcc doesn't comply about casts to char* */
 extern char* spd_strdup(char* string);
 
-
 /* == CONFIGURATION MANAGEMENT FUNCTIONS */
 
 /* Add dotconf configuration option */
@@ -153,27 +152,49 @@ free_config_options(configoption_t *opts, int *num)
        return NULL; \
    }    
 
+#define GLOBAL_SET_LOGLEVEL(name, arg, cond, str) \
+   DOTCONF_CB(cb_ ## name) \
+   { \
+       int val = cmd->data.value; \
+       if (cl_spec_section) \
+         FATAL("This command isn't allowed in a client specific section!"); \
+       if (!(cond)) FATAL(str); \
+       if (!SpeechdOptions.arg ## _set){ \
+         SpeechdOptions.arg = val; \
+         GlobalFDSet.arg = val; \
+       } \
+       return NULL; \
+   }
 
 /* == CALLBACK DEFINITIONS == */
-GLOBAL_FDSET_OPTION_CB_STR(DefaultModule, output_module);
-GLOBAL_FDSET_OPTION_CB_STR(DefaultLanguage, language);
-GLOBAL_FDSET_OPTION_CB_STR(DefaultClientName, client_name);
+GLOBAL_FDSET_OPTION_CB_STR(DefaultModule, output_module)
+GLOBAL_FDSET_OPTION_CB_STR(DefaultLanguage, language)
+GLOBAL_FDSET_OPTION_CB_STR(DefaultClientName, client_name)
 
-GLOBAL_FDSET_OPTION_CB_INT(DefaultRate, rate, (val>=-100)&&(val<=+100), "Rate out of range.");
-GLOBAL_FDSET_OPTION_CB_INT(DefaultPitch, pitch, (val>=-100)&&(val<=+100), "Pitch out of range.");
-GLOBAL_FDSET_OPTION_CB_INT(DefaultVolume, volume, (val>=-100)&&(val<=+100), "Volume out of range.");
-GLOBAL_FDSET_OPTION_CB_INT(DefaultSpelling, spelling_mode, 1, "Invalid spelling mode");
-GLOBAL_FDSET_OPTION_CB_INT(DefaultPauseContext, pause_context, 1, "");
+GLOBAL_FDSET_OPTION_CB_STR(AudioOutputMethod, audio_output_method)
+GLOBAL_FDSET_OPTION_CB_STR(AudioOSSDevice, audio_oss_device)
+GLOBAL_FDSET_OPTION_CB_STR(AudioALSADevice, audio_alsa_device)
+GLOBAL_FDSET_OPTION_CB_STR(AudioNASServer, audio_nas_server)
+GLOBAL_FDSET_OPTION_CB_STR(AudioPulseServer, audio_pulse_server)
+GLOBAL_FDSET_OPTION_CB_INT(AudioPulseMinLength, audio_pulse_min_length, 1, "")
 
-GLOBAL_FDSET_OPTION_CB_SPECIAL(DefaultPriority, priority, int, str2intpriority);
-GLOBAL_FDSET_OPTION_CB_SPECIAL(DefaultVoiceType, voice, EVoiceType, str2EVoice);
-GLOBAL_FDSET_OPTION_CB_SPECIAL(DefaultPunctuationMode, punctuation_mode, EPunctMode, str2EPunctMode);
-GLOBAL_FDSET_OPTION_CB_SPECIAL(DefaultCapLetRecognition, cap_let_recogn, ECapLetRecogn, str2ECapLetRecogn);
+GLOBAL_FDSET_OPTION_CB_INT(DefaultRate, rate, (val>=-100)&&(val<=+100), "Rate out of range.")
+GLOBAL_FDSET_OPTION_CB_INT(DefaultPitch, pitch, (val>=-100)&&(val<=+100), "Pitch out of range.")
+GLOBAL_FDSET_OPTION_CB_INT(DefaultVolume, volume, (val>=-100)&&(val<=+100), "Volume out of range.")
+GLOBAL_FDSET_OPTION_CB_INT(DefaultSpelling, spelling_mode, 1, "Invalid spelling mode")
+GLOBAL_FDSET_OPTION_CB_INT(DefaultPauseContext, pause_context, 1, "")
 
-SPEECHD_OPTION_CB_INT_M(LocalhostAccessOnly, localhost_access_only, val>=0, "Invalid access controll mode!");
-SPEECHD_OPTION_CB_INT_M(Port, port, val>=0, "Invalid port number!");
-SPEECHD_OPTION_CB_INT_M(LogLevel, log_level, (val>=0)&&(val<=5), "Invalid log (verbosity) level!");
-SPEECHD_OPTION_CB_INT(MaxHistoryMessages, max_history_messages, val>=0, "Invalid parameter!");
+GLOBAL_FDSET_OPTION_CB_SPECIAL(DefaultPriority, priority, int, str2intpriority)
+GLOBAL_FDSET_OPTION_CB_SPECIAL(DefaultVoiceType, voice, EVoiceType, str2EVoice)
+GLOBAL_FDSET_OPTION_CB_SPECIAL(DefaultPunctuationMode, punctuation_mode, EPunctMode, str2EPunctMode)
+GLOBAL_FDSET_OPTION_CB_SPECIAL(DefaultCapLetRecognition, cap_let_recogn, ECapLetRecogn, str2ECapLetRecogn)
+
+SPEECHD_OPTION_CB_STR(CommunicationMethod, communication_method)
+SPEECHD_OPTION_CB_STR(SocketName, socket_name)
+SPEECHD_OPTION_CB_INT_M(Port, port, val>=0, "Invalid port number!")
+SPEECHD_OPTION_CB_INT_M(LocalhostAccessOnly, localhost_access_only, val>=0, "Invalid access controll mode!")
+GLOBAL_SET_LOGLEVEL(LogLevel, log_level, (val>=0)&&(val<=5), "Invalid log (verbosity) level!")
+SPEECHD_OPTION_CB_INT(MaxHistoryMessages, max_history_messages, val>=0, "Invalid parameter!")
 
 DOTCONF_CB(cb_LanguageDefaultModule)
 {
@@ -335,6 +356,7 @@ DOTCONF_CB(cb_EndClient)
 }
 
 
+
 /* == CALLBACK FOR UNKNOWN OPTIONS == */
 
 DOTCONF_CB(cb_unknown)
@@ -343,6 +365,14 @@ DOTCONF_CB(cb_unknown)
     return NULL;
 }
 
+/* == Auto-spawn no-action callback == */
+DOTCONF_CB(cb_DisableAutoSpawn){
+  /* DisableAutoSpawn option is handled earlier during startup, not via the DotConf
+     mechanism. This callback here is a hack to ensure DotConf doesn't complain about
+     an unknown option */
+  return NULL;
+}
+
 
 /* == LOAD CALLBACKS == */
 
@@ -360,7 +390,10 @@ load_config_options(int *num_options)
 
     cl_spec_section = NULL;
    
+    ADD_CONFIG_OPTION(CommunicationMethod, ARG_STR);
+    ADD_CONFIG_OPTION(SocketName, ARG_STR);
     ADD_CONFIG_OPTION(Port, ARG_INT);
+    ADD_CONFIG_OPTION(DisableAutoSpawn, ARG_NONE);
     ADD_CONFIG_OPTION(LocalhostAccessOnly, ARG_INT);
     ADD_CONFIG_OPTION(LogFile, ARG_STR);
     ADD_CONFIG_OPTION(LogDir, ARG_STR);
@@ -381,6 +414,14 @@ load_config_options(int *num_options)
     ADD_CONFIG_OPTION(DefaultCapLetRecognition, ARG_STR);
     ADD_CONFIG_OPTION(DefaultPauseContext, ARG_INT);  
     ADD_CONFIG_OPTION(AddModule, ARG_LIST);
+
+    ADD_CONFIG_OPTION(AudioOutputMethod, ARG_STR);
+    ADD_CONFIG_OPTION(AudioOSSDevice, ARG_STR);
+    ADD_CONFIG_OPTION(AudioALSADevice, ARG_STR);
+    ADD_CONFIG_OPTION(AudioNASServer, ARG_STR);
+    ADD_CONFIG_OPTION(AudioPulseServer, ARG_STR);
+    ADD_CONFIG_OPTION(AudioPulseMinLength, ARG_INT);
+
     ADD_CONFIG_OPTION(BeginClient, ARG_STR);
     ADD_CONFIG_OPTION(EndClient, ARG_NONE);
     return options;
@@ -409,9 +450,26 @@ load_default_global_set_options()
     GlobalFDSet.ssml_mode = 0;
     GlobalFDSet.notification = NOTIFY_NOTHING;
 
+#ifdef __SUNPRO_C
+/* Added by Willie Walker - default to OSS for Solaris */
+    GlobalFDSet.audio_output_method = strdup("oss");
+#else
+    GlobalFDSet.audio_output_method = strdup("pulse,alsa");
+#endif /* __SUNPRO_C */
+    GlobalFDSet.audio_oss_device = strdup("/dev/dsp");
+    GlobalFDSet.audio_alsa_device = strdup("default");
+    GlobalFDSet.audio_nas_server = strdup("tcp/localhost:5450");
+    GlobalFDSet.audio_pulse_server = strdup("default");
+    GlobalFDSet.audio_pulse_min_length = 100;
+
     SpeechdOptions.max_history_messages = 10000;
 
+
+    /* Options which are accessible from command line must be handled
+       specially to make sure we don't overwrite them */
     if (!SpeechdOptions.log_level_set) SpeechdOptions.log_level = 3;    
+    if (!SpeechdOptions.communication_method) SpeechdOptions.communication_method = strdup("unix_socket");
+    if (!SpeechdOptions.socket_name) SpeechdOptions.socket_name = strdup("default");
     if (!SpeechdOptions.port_set) SpeechdOptions.port = SPEECHD_DEFAULT_PORT;
     if (!SpeechdOptions.localhost_access_only_set) SpeechdOptions.localhost_access_only = 1;
 
