@@ -305,9 +305,10 @@ get_messages_by_client(int uid){
 	GList *list = NULL;
 	GList *gl;
 	TSpeechDMessage* msg;
-	int i;
+	guint i;
+	guint history_length = g_list_length(message_history);
 
-	for (i=0;i<=g_list_length(message_history)-1;i++){
+	for (i = 0; i < history_length; i++) {
 		gl = g_list_nth(message_history, i);
 		assert(gl!=NULL);
 		msg = gl->data;
