@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: module.c,v 1.37 2008-02-08 10:01:09 hanke Exp $
+ * $Id: module.c,v 1.40 2008-07-07 14:30:51 hanke Exp $
  */
 
 #define _GNU_SOURCE
@@ -30,6 +30,13 @@
 #include "speechd.h"
 #include "output.h"
 
+#if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
+/* Added by Willie Walker - TEMP_FAILURE_RETRY, strndup, and getline
+ * are gcc-isms
+ */
+ssize_t getline (char **lineptr, size_t *n, FILE *f);
+#endif /* !(defined(__GLIBC__) || defined(_GNU_SOURCE)) */
+
 void
 destroy_module(OutputModule *module)
 {
@@ -192,6 +199,33 @@ load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile, char* mod_
 		}
 
 	    }
+
+	    if (SpeechdOptions.debug){
+	      MSG(4, "Switching debugging on for output module %s", module->name);
+	      output_module_debug(module);
+	    }
+	    /* Initialize audio settings */
+	    ret = output_send_audio_settings(module);
+	    if (ret !=0){
+	      MSG(1, "ERROR: Can't initialize audio in output module, see reason above.");
+	      module->working = 0;
+	      kill(module->pid, 9);
+	      waitpid(module->pid, NULL, WNOHANG);
+	      destroy_module(module);
+	      return NULL;
+	    }
+
+	    /* Send log level configuration setting */
+	    ret = output_send_loglevel_setting(module);
+	    if (ret !=0){
+	      MSG(1, "ERROR: Can't set the log level inin the output module.");
+	      module->working = 0;
+	      kill(module->pid, 9);
+	      waitpid(module->pid, NULL, WNOHANG);
+	      destroy_module(module);
+	      return NULL;
+	    }
+
 	    /* Get a list of supported voices */
 	    _output_get_voices(module);
 	    fclose(f);
@@ -261,3 +295,38 @@ reload_output_module(OutputModule *old_module)
 
     return 0;
 }
+
+
+int
+output_module_debug(OutputModule *module)
+{
+    char *new_log_path;
+
+    assert(module != NULL); assert(module->name != NULL);
+    if (!module->working) return -1;
+
+    MSG(4, "Output module debug logging for %s into %s", module->name,
+	SpeechdOptions.debug_destination);    
+    
+    new_log_path = g_strdup_printf("%s/%s.log", 
+				   SpeechdOptions.debug_destination,
+				   module->name);
+    
+    output_send_debug(module, 1, new_log_path);
+    
+    return 0;
+}
+
+
+int
+output_module_nodebug(OutputModule *module)
+{
+    assert(module != NULL); assert(module->name != NULL);
+    if (!module->working) return -1;
+
+    MSG(4, "Output module debug logging off for", module->name);
+    
+    output_send_debug(module, 0, NULL);
+    
+    return 0;
+}
