@@ -1,4 +1,4 @@
- /*
+/*
  * options.c - Parse and process possible command line options
  *
  * Copyright (C) 2003, 2006 Brailcom, o.p.s.
@@ -18,12 +18,14 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: options.c,v 1.11 2007-11-26 14:24:32 hanke Exp $
+ * $Id: options.c,v 1.13 2008-07-10 15:35:56 hanke Exp $
  */
 
 /* NOTE: Be careful not to include options.h, we would
    get repetitive initializations warnings */
 
+#include <sys/stat.h>
+
 #include "speechd.h"
 
 #include "options.h"
@@ -34,15 +36,19 @@ options_print_help(char *argv[])
     assert(argv);
     assert(argv[0]);
 
-    printf("Usage: %s [-{d|s}] [-l {1|2|3|4|5}] [-p=port] | [-v] | [-h]\n", argv[0]);
+    printf("Usage: %s [-{d|s}] [-l {1|2|3|4|5}] [-c com_method] [-S socket_name] [-p port] | [-v] | [-h]\n", argv[0]);
     printf("Speech Dispatcher -- Common interface for Speech Synthesis (GNU GPL)\n\n");
     printf("-d, --run-daemon     -      Run as a daemon\n"
     "-s, --run-single     -      Run as single application\n"
+    "-a, --spawn          -      Start only if autospawn is not disabled\n"
     "-l, --log-level      -      Set log level (1..5)\n"
-    "-p, --port           -      Specify a port number\n"
+    "-c, --communication-method  Communication method to use (unix_socket or inet_socket)\n"
+    "-S, --socket-name    -      Socket name to use for 'unix_socket' method (filesystem path or 'default')\n"
+    "-p, --port           -      Specify a port number for 'inet_socket' method\n"
     "-P, --pid-file       -      Set path to pid file\n"
     "-C, --config-dir     -      Set path to configuration\n"
     "-v, --version        -      Report version of this program\n"
+    "-D, --debug          -      Output debugging information into /tmp/.speech-dispatcher\n"
     "-h, --help           -      Print this info\n\n"
     "Copyright (C) 2003,2006 Brailcom, o.p.s.\n"
     "This is free software; you can redistribute it and/or modify it\n"
@@ -82,6 +88,11 @@ options_parse(int argc, char *argv[])
     int c_opt;
     int option_index;
     int val;
+    int ret;
+
+    char *tmpdir;
+    char *debug_logfile_path;
+	  
 
     assert (argc>0);
     assert(argv);
@@ -102,9 +113,18 @@ options_parse(int argc, char *argv[])
         case 'l':
             SPD_OPTION_SET_INT(log_level);	   
             break;
+        case 'c':
+            SPD_OPTION_SET_STR(communication_method);
+            break;
+        case 'S':
+            SPD_OPTION_SET_STR(socket_name);
+            break;
         case 'p':
             SPD_OPTION_SET_INT(port);
             break;
+        case 'a':
+   	    SpeechdOptions.spawn = TRUE;
+            break;
 	case 'P':
 	    SPD_OPTION_SET_STR(pid_file);
 	    break;
@@ -115,6 +135,36 @@ options_parse(int argc, char *argv[])
             options_print_version();
             exit(0);
             break;
+        case 'D':
+	  tmpdir = g_strdup(getenv("TMPDIR"));
+	  if (!tmpdir)
+	    tmpdir = g_strdup("/tmp");
+	  SpeechdOptions.debug_destination=g_strdup_printf("%s/speechd-debug", tmpdir);
+	  spd_free(tmpdir);
+
+	  ret = mkdir(SpeechdOptions.debug_destination, S_IRWXU);
+	  if (ret){
+	    MSG(1, "Can't create additional debug destination in %s, reason %d-%s",
+		SpeechdOptions.debug_destination, errno, strerror(errno));
+	    if (errno == EEXIST){
+	      MSG(1, "Debugging directory %s already exists, please delete it first",
+		  SpeechdOptions.debug_destination);
+	    }
+	    exit(1);
+	  }
+	    
+	  debug_logfile_path = g_strdup_printf("%s/speechd.log",
+					       SpeechdOptions.debug_destination);
+	  /* Open logfile for writing */
+	  debug_logfile = fopen(debug_logfile_path, "wx");
+	  spd_free(debug_logfile_path);
+	  if (debug_logfile == NULL){
+	    MSG(1, "Error: can't open additional debug logging file %s [%d-%s]!\n",
+		debug_logfile_path, errno, strerror(errno));
+	    exit(1);
+	  }
+	  SpeechdOptions.debug = 1;
+	  break;
         case 'h':
             options_print_help(argv);
             exit(0);
