@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: options.h,v 1.6 2007-11-21 14:19:13 hanke Exp $
+ * $Id: options.h,v 1.7 2008-07-07 14:31:08 hanke Exp $
  */
 
 #include <getopt.h>
@@ -26,16 +26,20 @@
 static struct option spd_long_options[] = {
     {"run-daemon", 0, 0, 'd'},
     {"run-single", 0, 0, 's'},
+    {"spawn", 0, 0, 'a'},
     {"log-level", 1, 0, 'l'},
+    {"communication-method", 1, 0, 'c'},
+    {"socket-name", 1, 0, 'S'},
     {"port", 1, 0, 'p'},
     {"pid-file", 1, 0, 'P'},
     {"config-file", 1, 0, 'C'},
     {"version", 0, 0, 'v'},
+    {"debug", 0, 0, 'D'},
     {"help", 0, 0, 'h'},
     {0, 0, 0, 0}
 };
 
-static char* spd_short_options = "dsl:p:P:C:vh";
+static char* spd_short_options = "dsal:c:S:p:P:C:vDh";
 
 void options_print_help(char *argv[]);
 void options_print_version(void);
