@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: output.c,v 1.34 2008-02-08 12:05:23 hanke Exp $
+ * $Id: output.c,v 1.38 2008-06-27 12:28:48 hanke Exp $
  */
 
 #include "output.h"
@@ -26,6 +26,88 @@
 #include "fdsetconv.c"
 #include "parse.h"
 
+#ifdef TEMP_FAILURE_RETRY	/* GNU libc */
+#define safe_write(fd, buf, count) TEMP_FAILURE_RETRY(write(fd, buf, count))
+#else /* TEMP_FAILURE_RETRY */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+static inline ssize_t
+safe_write(int fd, const void *buf, size_t count) {
+	do {
+		ssize_t w = write(fd, buf, count);
+
+		if (w == -1 && errno == EINTR) continue;
+		return w;
+	} while (1);
+}
+#endif /* TEMP_FAILURE_RETRY */
+
+#if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
+/* Added by Willie Walker - strndup, and getline are gcc-isms
+ */
+char *strndup ( const char *s, size_t n)
+{
+        size_t nAvail;
+        char *p;
+
+        if ( !s )
+                return 0;
+
+        if ( strlen(s) > n )
+                nAvail = n + 1;
+        else
+                nAvail = strlen(s) + 1;
+        p = malloc ( nAvail );
+        memcpy ( p, s, nAvail );
+        p[nAvail - 1] = '\0';
+
+        return p;
+}
+
+#define BUFFER_LEN 256
+ssize_t getline (char **lineptr, size_t *n, FILE *f)
+{
+        char ch;
+        size_t m = 0;
+        ssize_t buf_len = 0;
+        char * buf = NULL;
+        char * p = NULL;
+
+	if (errno != 0) {
+	        errno = 0;
+	}
+        while ( (ch = getc(f)) !=EOF )
+        {
+                if (errno != 0)
+                        return -1;
+                if ( m++ >= buf_len )
+                {
+                        buf_len += BUFFER_LEN;
+                        buf = (char *) realloc(buf, buf_len + 1);
+                        if ( buf == NULL )
+                        {
+                                return -1;
+                        }
+                        p = buf + buf_len - BUFFER_LEN;
+                }
+                *p = ch;
+                p++;
+                if ( ch == '\n' )
+                        break;
+        }
+        if ( m == 0 )
+        {
+                return -1;
+        } else {
+                *p = '\0';
+                *lineptr = buf;
+                *n = m;
+                return m;
+        }
+}
+#endif /* !(defined(__GLIBC__) || defined(_GNU_SOURCE)) */
+
 void
 output_set_speaking_monitor(TSpeechDMessage *msg, OutputModule *output)
 {
@@ -45,23 +127,64 @@ get_output_module_by_name(char *name)
   return output;
 }
 
+/* get_output_module tries to return a pointer to the
+   appropriate output module according to message context.
+   If it is not possible to find the required module,
+   it will subsequently try to get the default module,
+   any of the other remaining modules except dummy and
+   at last, the dummy output module.
+
+   Only if not even dummy output module is working
+   (serious issues), it will log an error message and return
+   a NULL pointer.
+
+*/
+
 OutputModule*
 get_output_module(const TSpeechDMessage *message)
 {
   OutputModule *output = NULL;
-  
+  GList *gl;
+  int i;
+
   if (message->settings.output_module != NULL){
     MSG(5, "Desired output module is %s", message->settings.output_module);
     output = get_output_module_by_name(message->settings.output_module);
-    if(output == NULL){
+    if((output == NULL) || !output->working){
+      // If the requested module was not found or is not working,
+      // first try to use the default output module
       if (GlobalFDSet.output_module != NULL){
 	MSG(3,"Warning: Didn't find prefered output module, using default");                
-	output = g_hash_table_lookup(output_modules, GlobalFDSet.output_module); 
-	if (output == NULL || !output->working) 
-	  MSG(2, "Error: Can't find default output module or it's not working!");
+	output = get_output_module_by_name(GlobalFDSet.output_module); 
       }
+      if (output == NULL || !output->working){
+	MSG(3, "Couldn't load default output module, trying other modules");
+      }
+
+      // Try all other output modules now to see if some of them
+      // is working
+      MSG(3, "Trying other output modules");
+      for (i=0; i<=g_list_length(output_modules_list)-1; i++){
+	gl = g_list_nth(output_modules_list, i);
+	if (!gl || !gl->data) break;
+	if (strcmp(gl->data, "dummy"))
+	  output = get_output_module_by_name(gl->data);
+	if ((output != NULL) && (output->working)){
+	  MSG(3, "Output module %s seems to be working, using it", gl->data);
+	  break;
+	}
+      }
+
+      // Did we get something working? If not, use dummy (it will just play
+      // a pre-synthesized error message with some hints over and over).
+      if (output == NULL || !output->working){
+	MSG(1, "Error: No output module seems to be working, using the dummy output module");
+	output = get_output_module_by_name("dummy");
+      }
+
+      // Give up....
       if (output == NULL){
-	MSG(3, "Error: Unspecified output module!\n");
+	MSG(1, "Error: No output module working, not even dummy, no sound produced!\n");
 	return NULL;
       }
     }
@@ -84,14 +207,14 @@ static output_unlock(void)
   {  output_unlock(); \
     return (value); }
 
-char*
+GString*
 output_read_reply(OutputModule *output)
 {
     GString *rstr;
     int bytes;
     char *line = NULL;
     size_t N = 0;
-    char *reply;
+    gboolean errors = FALSE;
     
     rstr = g_string_new("");
     
@@ -104,18 +227,23 @@ output_read_reply(OutputModule *output)
 	    output->working = 0;
 	    speaking_module = NULL;
 	    output_check_module(output);
-	    return NULL; /* Broken pipe */   
+	    errors = TRUE; /* Broken pipe */   
+	} else {
+	    MSG(5, "Got %d bytes from output module over socket", bytes);
+	    g_string_append(rstr, line);
 	}
-	MSG(5, "Got %d bytes from output module over socket", bytes);
-	g_string_append(rstr, line);
 	/* terminate if we reached the last line (without '-' after numcode) */
-    }while( !((strlen(line) < 4) || (line[3] == ' ')));
+    }while( !errors && !((strlen(line) < 4) || (line[3] == ' ')));
 
-    /* The resulting message received from the socket is stored in reply */
-    reply = rstr->str;
-    g_string_free(rstr, FALSE);
+    if(line != NULL)
+	free(line);	/* getline allocates with malloc and realloc. */
 
-    return reply;
+    if(errors) {
+	g_string_free(rstr, TRUE);
+	rstr = NULL;
+    }
+
+    return rstr;
 }
 
 char*
@@ -138,12 +266,12 @@ int
 output_send_data(char* cmd, OutputModule *output, int wfr)
 {
     int ret;
-    char *response;
+    GString *response;
 
     if (output == NULL) return -1;
     if (cmd == NULL) return -1;
     
-    ret = TEMP_FAILURE_RETRY(write(output->pipe_in[1], cmd, strlen(cmd)));
+    ret = safe_write(output->pipe_in[1], cmd, strlen(cmd));
     fflush(NULL);
     if (ret == -1){
         MSG(2, "Error: Broken pipe to module.");        
@@ -155,30 +283,33 @@ output_send_data(char* cmd, OutputModule *output, int wfr)
     MSG2(5, "output_module", "Command sent to output module: |%s| (%d)", cmd, wfr);
     
     if (wfr){                   /* wait for reply? */	
+	int ret = 0;
 	response = output_read_reply(output);
 	if (response == NULL) return -1;
 
-        MSG2(5, "output_module", "Reply from output module: |%s|", response);
+        MSG2(5, "output_module", "Reply from output module: |%s|", response->str);
 
-        if (response[0] == '3'){
-            MSG(2, "Error: Module reported error in request from speechd (code 3xx).");
-	    spd_free(response);
-            return -2; /* User (speechd) side error */
-        }
+	switch (response->str[0]){
+	    case '3':
+		MSG(2, "Error: Module reported error in request from speechd (code 3xx): %s.", response->str);
+		ret = -2; /* User (speechd) side error */
+		break;
 
-        if (response[0] == '4'){
-            MSG(2, "Error: Module reported error in itself (code 4xx).");
-	    spd_free(response);
-            return -3; /* Module side error */
-        }
+	    case '4':
+		MSG(2, "Error: Module reported error in itself (code 4xx): %s", response->str);
+		ret = -3; /* Module side error */
+		break;
 
-        if (response[0] == '2'){
-	    return 0;
-        }else{                  /* unknown response */
-            MSG(3, "Unknown response from output module!");
-	    spd_free(response);
-            return -3;
-        }
+	    case '2':
+		ret = 0;
+		break;
+	    default:                  /* unknown response */
+		MSG(3, "Unknown response from output module!");
+		ret = -3;
+		break;
+	}
+    g_string_free(response, TRUE);
+    return ret;
     }       
     
     return 0;
@@ -188,16 +319,18 @@ int
 _output_get_voices(OutputModule *module)
 {
   VoiceDescription** voice_dscr;
-  char *reply;
+  GString *reply;
   gchar **lines;
   gchar **atoms;
   int i;
+  int ret = 0;
+  gboolean errors = FALSE;
 
   output_lock();
 
   if (module == NULL){
     MSG(1, "ERROR: Can't list voices for broken output module");
-    return -1;
+    OL_RET(-1);
   }
   output_send_data("LIST VOICES\n", module, 0);
   reply = output_read_reply(module);
@@ -209,38 +342,44 @@ _output_get_voices(OutputModule *module)
   }
 
   //TODO: only 256 voices supported here
-  lines = g_strsplit(reply, "\n", 256);
+  lines = g_strsplit(reply->str, "\n", 256);
+  g_string_free(reply, TRUE);
   voice_dscr = malloc(256*sizeof(VoiceDescription*));
-  for (i=0; ;i++){
-    if (lines[i] == NULL) break;
+  for (i = 0; !errors && (lines[i] != NULL); i++) {
     MSG(1, "LINE here:|%s|", lines[i]);
     if (strlen(lines[i])<=4){
       MSG(1, "ERROR: Bad communication from driver in synth_voices");
-      output_unlock();
-      return -1;
+      ret = -1;
+      errors = TRUE;
     }
-    if (lines[i][3] == ' ') break;
+    else if (lines[i][3] == ' ')
+      break;
     else if (lines[i][3] == '-'){
       atoms = g_strsplit(&lines[i][4]," ", 0);
       // Name, language, dialect
       if ((atoms[0] == NULL) || (atoms[1] == NULL) || (atoms[2] == NULL)){
-	output_unlock();
-	return -1;
+	ret = -1;
+      errors = TRUE;
+      } else {
+        //Fill in VoiceDescription
+        voice_dscr[i] = (VoiceDescription*) malloc(sizeof(VoiceDescription));
+        voice_dscr[i]->name=strdup(atoms[0]);
+        voice_dscr[i]->language=strdup(atoms[1]);
+        voice_dscr[i]->dialect=strdup(atoms[2]);
       }
-      //Fill in VoiceDescription
-      voice_dscr[i] = (VoiceDescription*) malloc(sizeof(VoiceDescription));
-      voice_dscr[i]->name=strdup(atoms[0]);
-      voice_dscr[i]->language=strdup(atoms[1]);
-      voice_dscr[i]->dialect=strdup(atoms[2]);
+    if (atoms != NULL)
+      g_strfreev(atoms);
     }
+    /* Should we do something in a final "else" branch? */
 
   }
   voice_dscr[i] = NULL;
+  g_strfreev(lines);
 
   module->voices=voice_dscr;
 
   output_unlock();
-  return 0;
+  return ret;
 }
 
 VoiceDescription**
@@ -324,6 +463,89 @@ output_send_settings(TSpeechDMessage *msg, OutputModule *output)
 #undef ADD_SET_STR
 
 
+#define ADD_SET_INT(name) \
+    g_string_append_printf(set_str, #name"=%d\n", GlobalFDSet.name);
+#define ADD_SET_STR(name) \
+    if (GlobalFDSet.name != NULL){ \
+       g_string_append_printf(set_str, #name"=%s\n", GlobalFDSet.name); \
+    }else{ \
+       g_string_append_printf(set_str, #name"=NULL\n"); \
+    }
+
+int
+output_send_audio_settings(OutputModule *output)
+{
+    GString *set_str;
+    int err;
+
+    MSG(4, "Module set parameters.");
+    set_str = g_string_new("");
+    ADD_SET_STR(audio_output_method);
+    ADD_SET_STR(audio_oss_device);
+    ADD_SET_STR(audio_alsa_device);
+    ADD_SET_STR(audio_nas_server);
+    ADD_SET_STR(audio_pulse_server);
+    ADD_SET_INT(audio_pulse_min_length);
+
+    SEND_CMD_N("AUDIO");
+    SEND_DATA_N(set_str->str);
+    SEND_CMD_N(".");
+
+    g_string_free(set_str, 1);
+
+    return 0;
+}
+
+int
+output_send_loglevel_setting(OutputModule *output)
+{
+    GString *set_str;
+    int err;
+
+    MSG(4, "Module set parameters.");
+    set_str = g_string_new("");
+    ADD_SET_INT(log_level);
+
+    SEND_CMD_N("LOGLEVEL");
+    SEND_DATA_N(set_str->str);
+    SEND_CMD_N(".");
+
+    g_string_free(set_str, 1);
+
+    return 0;
+}
+#undef ADD_SET_INT
+#undef ADD_SET_STR
+
+int
+output_send_debug(OutputModule *output, int flag, char* log_path)
+{
+    char *cmd_str;
+    int err;
+
+    MSG(4, "Module sending debug flag %d with file %s", flag, log_path);
+    
+    output_lock();
+    if (flag){
+      cmd_str = g_strdup_printf("DEBUG ON %s \n", log_path);
+      err = output_send_data(cmd_str, output, 1);
+      spd_free(cmd_str);
+      if (err){
+	MSG(3, "ERROR: Can't set debugging on for output module %s", output->name);
+	OL_RET(-1);
+      }      
+    }else{
+      err = output_send_data("DEBUG OFF \n", output, 1);
+      if (err){
+	MSG(3, "ERROR: Can't switch debugging off for output module %s", output->name);
+	OL_RET(-1);
+      }
+      
+    }
+   
+    OL_RET(0);
+}
+
 int
 output_speak(TSpeechDMessage *msg)
 {
@@ -406,7 +628,8 @@ output_pause()
 int
 output_module_is_speaking(OutputModule *output, char **index_mark)
 {
-    char *response;
+    GString *response;
+    int retcode = -1;
 
     output_lock();
 
@@ -423,59 +646,74 @@ output_module_is_speaking(OutputModule *output, char **index_mark)
 	OL_RET(-1);
     }    
 
-    MSG2(5, "output_module", "Reply from output module: |%s|", response);
+    MSG2(5, "output_module", "Reply from output module: |%s|", response->str);
 
-    if (strlen(response) < 4){
+    if (response->len < 4){
 	MSG2(2, "output_module",
 	     "Error: Wrong communication from output module! Reply less than four bytes.");
+	g_string_free(response, TRUE);
 	OL_RET(-1); 
     }
 
-    if (response[0] == '3'){
-	MSG(2, "Error: Module reported error in request from speechd (code 3xx).");
-	OL_RET(-2); /* User (speechd) side error */
-    }
-    else if (response[0] == '4'){
-	MSG(2, "Error: Module reported error in itself (code 4xx).");
-	OL_RET(-3); /* Module side error */
-    }
-    else if (response[0] == '2'){
-	if (strlen(response) > 4){
-	    if (response[3] == '-'){
-		char *p;
-		p = strchr(response, '\n');
-		*index_mark = (char*) strndup(response+4, p-response-4);
-		MSG2(5, "output_module", "Detected INDEX MARK: %s", *index_mark);
-	    }else{
-		MSG2(2, "output_module", "Error: Wrong communication from output module!"
-		    "Reply on SPEAKING not multi-line.");
-		OL_RET(-1); 
+    switch(response->str[0]) {
+    	case '3':
+	    MSG(2, "Error: Module reported error in request from speechd (code 3xx).");
+	    retcode = -2; /* User (speechd) side error */
+	    break;
+
+	case '4':
+	    MSG(2, "Error: Module reported error in itself (code 4xx).");
+	    retcode = -3; /* Module side error */
+	    break;
+
+	case '2':
+	    retcode = 0;
+	    if (response->len > 4){
+		if (response->str[3] == '-'){
+		    char *p;
+		    p = strchr(response->str, '\n');
+		    *index_mark = (char*) strndup(response->str+4, p-response->str-4);
+		    MSG2(5, "output_module", "Detected INDEX MARK: %s", *index_mark);
+		}else{
+		    MSG2(2, "output_module", "Error: Wrong communication from output module!"
+			"Reply on SPEAKING not multi-line.");
+		    retcode = -1; 
+		}
 	    }
-	}
-	OL_RET(0);
-    }else if(response[0] == '7'){
-	MSG2(5, "output_module", "Received event:\n %s", response);
-	if (!strncmp(response, "701", 3)) *index_mark = (char*) strdup("__spd_begin");
-	else if (!strncmp(response, "702", 3)) *index_mark = (char*) strdup("__spd_end");
-	else if (!strncmp(response, "703", 3)) *index_mark = (char*) strdup("__spd_stopped");
-	else if (!strncmp(response, "704", 3)) *index_mark = (char*) strdup("__spd_paused");
-	else if (!strncmp(response, "700", 3)){
-	    char *p;
-	    p = strchr(response, '\n');
-	    MSG2(5, "output_module", "response:|%s|\n p:|%s|", response, p);
-	    *index_mark = (char*) strndup(response+4, p-response-4);
-	    MSG2(5, "output_module", "Detected INDEX MARK: %s", *index_mark);
-	}else{
-	    MSG2(2, "output_module", "ERROR: Unknown event received from output module");
-	    OL_RET(-5);
-	}
-	OL_RET(0);
-    }else{                  /* unknown response */
-	MSG(3, "Unknown response from output module!");
-	OL_RET(-3);
+	    break;
+
+	case '7':
+	    retcode = 0;
+	    MSG2(5, "output_module", "Received event:\n %s", response->str);
+	    if (!strncmp(response->str, "701", 3))
+		*index_mark = (char*) strdup("__spd_begin");
+	    else if (!strncmp(response->str, "702", 3))
+		*index_mark = (char*) strdup("__spd_end");
+	    else if (!strncmp(response->str, "703", 3))
+		*index_mark = (char*) strdup("__spd_stopped");
+	    else if (!strncmp(response->str, "704", 3))
+		*index_mark = (char*) strdup("__spd_paused");
+	    else if (!strncmp(response->str, "700", 3)) {
+		char *p;
+		p = strchr(response->str, '\n');
+		MSG2(5, "output_module", "response:|%s|\n p:|%s|", response->str, p);
+		*index_mark = (char*) strndup(response->str+4, p-response->str-4);
+		MSG2(5, "output_module", "Detected INDEX MARK: %s", *index_mark);
+	    } else {
+		MSG2(2, "output_module", "ERROR: Unknown event received from output module");
+		retcode = -5;
+	    }
+	    break;
+
+	default:                  /* unknown response */
+	    MSG(3, "Unknown response from output module!");
+	    retcode = -3;
+	    break;
+
     }
 
-    OL_RET(-1)
+    g_string_free(response, TRUE);
+    OL_RET(retcode)
 }
 
 int
@@ -667,5 +905,3 @@ escape_dot(char *otext)
 
     return ret;
 }
-
-
