@@ -23,7 +23,8 @@
 
 #include <gmodule.h>
 #include <sys/stat.h>
-
+#include <sys/socket.h>
+#include <sys/un.h>
 
 #include "speechd.h"
 
@@ -50,6 +51,44 @@ int server_socket;
 
 void speechd_load_configuration(int sig);
 
+#ifdef __SUNPRO_C
+/* Added by Willie Walker - daemon is a gcc-ism
+ */
+#include <sys/filio.h>
+static int daemon(int nochdir, int noclose)
+{
+       int fd, i;
+
+       switch (fork()) {
+               case 0:
+                       break;
+               case -1:
+                       return -1;
+               default:
+                       _exit(0);
+       }
+
+       if (!nochdir) {
+               chdir("/");
+       }
+
+       if (setsid() < 0) {
+               return -1;
+       }
+
+       if (!noclose) {
+               if (fd = open("/dev/null", O_RDWR) >= 0) {
+                       for (i = 0; i < 3; i++) {
+                               dup2(fd, i);
+                       }
+                       if (fd > 2) {
+                               close(fd);
+                       }
+               }
+       }
+       return 0;
+}
+#endif /* __SUNPRO_C */
 
 char*
 spd_get_path(char *filename, char* startdir)
@@ -439,7 +478,10 @@ speechd_modules_nodebug(void)
 void
 speechd_options_init(void)
 {
+    SpeechdOptions.spawn = FALSE;
     SpeechdOptions.log_level_set = 0;
+    SpeechdOptions.communication_method = NULL;
+    SpeechdOptions.socket_name = NULL;
     SpeechdOptions.port_set = 0;
     SpeechdOptions.localhost_access_only_set = 0;
     SpeechdOptions.pid_file = NULL;
@@ -528,22 +570,13 @@ speechd_init()
     if(ret != 0) DIE("Mutex initialization failed");
 
     if (SpeechdOptions.log_dir == NULL){
-      if (SpeechdOptions.home_speechd_dir){
-	SpeechdOptions.log_dir = g_strdup_printf("%s/log/",
-						 SpeechdOptions.home_speechd_dir);
-	mkdir(SpeechdOptions.log_dir, S_IRWXU);
-	if (!SpeechdOptions.debug_destination){
-	  SpeechdOptions.debug_destination = g_strdup_printf("%slog/debug",
-							     SpeechdOptions.home_speechd_dir);
-	  mkdir(SpeechdOptions.debug_destination, S_IRWXU);
-	}
-      }else{
-	SpeechdOptions.log_dir = strdup("/var/log/speech-dispatcher/");
-	if (!SpeechdOptions.debug_destination){
-	  SpeechdOptions.debug_destination = strdup("/var/log/speech-dispatcher/debug");
-	  mkdir(SpeechdOptions.debug_destination, S_IRWXU);
-	}
-      }
+      SpeechdOptions.log_dir = g_strdup_printf("%s/log/", SpeechdOptions.home_speechd_dir);
+      mkdir(SpeechdOptions.log_dir, S_IRWXU);
+      if (!SpeechdOptions.debug_destination){
+	SpeechdOptions.debug_destination = g_strdup_printf("%slog/debug",
+							   SpeechdOptions.home_speechd_dir);
+	mkdir(SpeechdOptions.debug_destination, S_IRWXU);
+      }     
     }
 
     /* Load configuration from the config file*/
@@ -574,8 +607,7 @@ speechd_load_configuration(int sig)
 
     /* Clean previous configuration */
     assert(output_modules != NULL);
-    g_hash_table_foreach_remove(output_modules, speechd_modules_terminate,
-                                NULL);
+    g_hash_table_foreach_remove(output_modules, speechd_modules_terminate, NULL);
 
     /* Make sure there aren't any more child processes left */
     while(waitpid(-1, NULL, WNOHANG) > 0);    
@@ -591,17 +623,17 @@ speechd_load_configuration(int sig)
                                     NULL, NULL, 0);
     
     configfile = dotconf_create(SpeechdOptions.conf_file, spd_options, 0, CASE_INSENSITIVE);
-    if (!configfile){
+    if (configfile){
+      configfile->includepath = strdup(SpeechdOptions.conf_dir);
+      MSG(5, "Config file include path is: %s", configfile->includepath);
+      if (dotconf_command_loop(configfile) == 0) DIE("Error reading config file\n");
+      dotconf_cleanup(configfile);
+      MSG(2,"Configuration has been read from \"%s\"", SpeechdOptions.conf_file);
+    }else{
       MSG(1, "Can't open %s", SpeechdOptions.conf_file);
-      DIE ("Error opening config file\n");
     }
-    configfile->includepath = strdup(SpeechdOptions.conf_dir);
-    MSG(5, "Config file include path is: %s", configfile->includepath);
-    if (dotconf_command_loop(configfile) == 0) DIE("Error reading config file\n");
-    dotconf_cleanup(configfile);
 
     free_config_options(spd_options, &spd_num_options);
-    MSG(2,"Configuration has been read from \"%s\"", SpeechdOptions.conf_file);
 }
 	
 void
@@ -667,13 +699,13 @@ create_pid_file()
         /* If there is a lock, exit, otherwise remove the old file */
         ret = fcntl(pid_fd, F_GETLK, &lock);
         if (ret == -1){
-            fprintf(stderr, "Can't check lock status of an existing pid file.\n");
+            MSG(1, "Can't check lock status of an existing pid file.\n");
             return -1;
         }
 
         fclose(pid_file);
         if (lock.l_type != F_UNLCK){
-            fprintf(stderr, "Speech Dispatcher already running.\n");
+            MSG(2, "Speech Dispatcher already running.\n");
             return -1;
         }
 
@@ -683,8 +715,8 @@ create_pid_file()
     /* Create a new pid file and lock it */
     pid_file = fopen(SpeechdOptions.pid_file, "w");
     if (pid_file == NULL){
-        fprintf(stderr, "Can't create pid file in %s, wrong permissions?\n",
-                SpeechdOptions.pid_file);
+        MSG(1, "Can't create pid file in %s, wrong permissions?\n",
+	    SpeechdOptions.pid_file);
         return -1;
     }
     fprintf(pid_file, "%d\n", getpid());
@@ -698,7 +730,7 @@ create_pid_file()
 
     ret = fcntl(pid_fd, F_SETLK, &lock);
     if (ret == -1){
-        fprintf(stderr, "Can't set lock on pid file.\n");
+        MSG(1, "Can't set lock on pid file.\n");
         return -1;
     }
 
@@ -718,34 +750,108 @@ logging_init(void)
   assert(file_name != NULL);
   if (!strncmp(file_name,"stdout",6)){
     logfile = stdout;
-    return;
-  }
-  if (!strncmp(file_name,"stderr",6)){
+  } else if (!strncmp(file_name, "stderr", 6)) {
     logfile = stderr;
-    return;
-  }
-  logfile = fopen(file_name, "a");
-  if (logfile == NULL){
-    fprintf(stderr, "Error: can't open logging file %s! Using stdout.\n",
-	    file_name);
-    logfile = stdout;
+  } else {
+    logfile = fopen(file_name, "a");
+    if (logfile == NULL){
+      fprintf(stderr, "Error: can't open logging file %s! Using stdout.\n",
+	      file_name);
+      logfile = stdout;
+    } else {
+      MSG(2,"Speech Dispatcher Logging to file %s", file_name);
+    }
   }
 
   if (!debug_logfile)
     debug_logfile = stdout;
 
-  MSG(2,"Speech Dispatcher Logging to file %s", file_name);
+  g_free(file_name);
   return;
-
 }
 
+/* --- Sockets --- */
+
+int
+make_local_socket (const char *filename)
+{
+  struct sockaddr_un name;
+  int sock;
+  size_t size;
+
+  /* Create the socket. */
+  sock = socket (AF_UNIX, SOCK_STREAM, 0);
+  if (sock < 0){
+    FATAL("Can't create local socket");
+  }
+
+  /* Bind a name to the socket. */
+  name.sun_family = AF_UNIX;
+  strncpy (name.sun_path, filename, sizeof (name.sun_path));
+  name.sun_path[sizeof (name.sun_path) - 1] = '\0';
+  size = SUN_LEN(&name);
+
+  if (bind (sock, (struct sockaddr *) &name, size) < 0){
+    FATAL("Can't bind local socket");
+  }
+
+  if (listen(sock, 50) == -1){
+    MSG(2,"ERRNO:%s", strerror(errno));
+    FATAL("listen() failed for local socket");
+  }
+
+  return sock;
+}
+
+int
+make_inet_socket(const int port)
+{
+  struct sockaddr_in server_address;    
+  int server_socket;
+
+  /* Create an inet socket */
+  server_socket = socket(AF_INET, SOCK_STREAM, 0);
+  if (server_socket < 0){
+      FATAL("Can't create inet socket");
+  }
+
+  /* Set REUSEADDR flag */
+  const int flag = 1;
+  if (setsockopt(server_socket, SOL_SOCKET, SO_REUSEADDR, &flag,
+		 sizeof (int)))
+    MSG(2,"Error: Setting socket option failed!");
+  
+  server_address.sin_family = AF_INET;
+  
+  /* Enable access only to localhost or for any address
+     based on LocalhostAccessOnly configuration option. */
+  if (SpeechdOptions.localhost_access_only)
+    server_address.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+  else
+    server_address.sin_addr.s_addr = htonl(INADDR_ANY);
+  
+  server_address.sin_port = htons(port);
+
+  MSG(3,"Openning inet socket connection");
+  if (bind(server_socket, (struct sockaddr *)&server_address,
+	   sizeof(server_address)) == -1){
+    MSG(1, "bind() failed: %s", strerror(errno));
+    FATAL("Couldn't open inet socket, try a few minutes later.");
+  }
+
+  if (listen(server_socket, 50) == -1){
+    MSG(2,"ERRNO:%s", strerror(errno));
+    FATAL("listen() failed for inet socket, another Speech Dispatcher running?");
+  }
+
+  return server_socket;
+}
 
 /* --- MAIN --- */
 
 int
 main(int argc, char *argv[])
 {
-    struct sockaddr_in server_address;    
     fd_set testfds;
     int fd;
     int ret;
@@ -768,30 +874,70 @@ main(int argc, char *argv[])
 
     MSG(1, "Speech Dispatcher "VERSION" starting");
 
-    /* Check if there is .speech-dispatcher directory
-       in user's home directory. If yes, put everything
-       here */
+    /* By default, search for configuration options and put everything
+       in a .speech-dispatcher directory  in user's home directory. */
     {
-      char *home_dir;
-      GDir *testing;
-      home_dir = (char*) g_get_home_dir();
-      if (home_dir){
-	SpeechdOptions.home_speechd_dir = g_strdup_printf("%s/.speech-dispatcher/",
-							  home_dir);
+      const char *user_home_dir;
+      
+      /* Get users home dir */
+      user_home_dir = g_getenv ("HOME");
+      if (!user_home_dir)
+	user_home_dir = g_get_home_dir ();
+      if (user_home_dir){
+	/* Setup a ~/.speechd-dispatcher/ directory or create a new one */
+	SpeechdOptions.home_speechd_dir = g_strdup_printf("%s/.speech-dispatcher", user_home_dir);
 	MSG(4, "Home dir found, trying to find %s", SpeechdOptions.home_speechd_dir);
-	testing =  g_dir_open(SpeechdOptions.home_speechd_dir, 0, NULL);
-	if (testing){
-	  /* Ok, directory exists */
-	  g_dir_close(testing);
-	  MSG(4, "Using home directory: %s for configuration, pidfile and logging",
-	      SpeechdOptions.home_speechd_dir);
-	}else{
-	  MSG(1, "Directory .speech-dispatcher not found in home");
-	  g_free(SpeechdOptions.home_speechd_dir);
-	  SpeechdOptions.home_speechd_dir = NULL;
+	g_mkdir(SpeechdOptions.home_speechd_dir, S_IRWXU);
+	MSG(4, "Using home directory: %s for configuration, pidfile and logging", SpeechdOptions.home_speechd_dir);
+	/* Pidfile */
+	if (SpeechdOptions.pid_file == NULL){
+	  /* If no pidfile path specified on command line, use default local dir */
+	  SpeechdOptions.pid_file = g_strdup_printf("%s/pid/speech-dispatcher.pid", SpeechdOptions.home_speechd_dir);
+	  g_mkdir(g_path_get_dirname(SpeechdOptions.pid_file), S_IRWXU);
+	}       
+	/* Config file */
+	if (SpeechdOptions.conf_dir == NULL){
+	  /* If no conf_dir was specified on command line, try default local config dir */
+	  SpeechdOptions.conf_dir = g_strdup_printf("%s/conf/", SpeechdOptions.home_speechd_dir);
+	  if (!g_file_test(SpeechdOptions.conf_dir, G_FILE_TEST_IS_DIR)){
+	    /* If the local confiration dir doesn't exist, read the global configuration */
+	    if (strcmp(SYS_CONF, "")) SpeechdOptions.conf_dir = strdup(SYS_CONF);
+	    else SpeechdOptions.conf_dir = strdup("/etc/speech-dispatcher/");
+	  }
 	}
-	g_free(home_dir);
+      }else{
+	if (SpeechdOptions.pid_file == NULL || SpeechdOptions.conf_dir == NULL)
+	  FATAL("Paths to pid file or conf dir not specified and the current user has no HOME directory!");
       }
+      SpeechdOptions.conf_file = g_strdup_printf("%s/speechd.conf", SpeechdOptions.conf_dir);      
+    }
+
+    /* Check for PID file or create a new one or exit if Speech Dispatcher
+       is already running */
+    if (create_pid_file() != 0) exit(1);
+
+    /* Handle --spawn request */
+    if (SpeechdOptions.spawn){
+      /* Check whether spawning is not disabled */
+      gchar *config_contents;
+      int err;
+      GRegex *regexp;
+      int result;
+
+      err = g_file_get_contents(SpeechdOptions.conf_file, &config_contents, NULL, NULL);
+      if (err == FALSE){
+	MSG(1,"Error openning %s", SpeechdOptions.conf_file);
+	FATAL("Can't open conf file");
+      }
+      regexp = g_regex_new ("^[ ]*DisableAutoSpawn", G_REGEX_MULTILINE,0,NULL);
+      result = g_regex_match(regexp, config_contents, 0, NULL);      
+      if (result){
+	MSG(4,"Autospawn requested but disabled in configuration");	
+	exit(1);
+      }
+      g_free(config_contents);
+      g_regex_unref(regexp);
+      MSG(2,"Starting Speech Dispatcher due to auto-spawn");
     }
 
     /* Initialize logging mutex to workaround ctime threading bug */
@@ -801,33 +947,7 @@ main(int argc, char *argv[])
       fprintf(stderr, "Mutex initialization failed");
       exit(1);
     }
-
-    if (SpeechdOptions.pid_file == NULL){
-      if (SpeechdOptions.home_speechd_dir){	
-	SpeechdOptions.pid_file = g_strdup_printf("%s/pid/speech-dispatcher.pid",
-						  SpeechdOptions.home_speechd_dir);
-	mkdir(g_path_get_dirname(SpeechdOptions.pid_file), S_IRWXU);
-      }else if (!strcmp(PIDPATH, ""))
-	SpeechdOptions.pid_file = strdup("/var/run/speech-dispatcher.pid");
-      else
-	SpeechdOptions.pid_file = strdup(PIDPATH"speech-dispatcher.pid");
-    }
     
-    if (SpeechdOptions.conf_dir == NULL){
-      if (SpeechdOptions.home_speechd_dir){
-	SpeechdOptions.conf_dir = g_strdup_printf("%sconf/",
-						  SpeechdOptions.home_speechd_dir);
-      }else if (!strcmp(SYS_CONF, "")){
-	SpeechdOptions.conf_dir = strdup("/etc/speech-dispatcher/");
-      }else{
-	SpeechdOptions.conf_dir = strdup(SYS_CONF);
-      }
-    }
-    SpeechdOptions.conf_file = g_strdup_printf("%s/speechd.conf",
-					       SpeechdOptions.conf_dir);
-    /* This is needed for DotConf so that it knows where to include files from */
-
-    if (create_pid_file() != 0) exit(1);
 
     /* Register signals */
     (void) signal(SIGINT, speechd_quit);	
@@ -837,43 +957,42 @@ main(int argc, char *argv[])
 
     speechd_init();
 
-    MSG(1, "Speech Dispatcher will use port %d", SpeechdOptions.port);
+    if(!strcmp(SpeechdOptions.communication_method, "inet_socket")){
+      MSG(2, "Speech Dispatcher will use inet port %d", SpeechdOptions.port);
+      /* Connect and start listening on inet socket */
+      server_socket = make_inet_socket(SpeechdOptions.port);
+    }else if (!strcmp(SpeechdOptions.communication_method, "unix_socket")){
+      /* Determine appropariate socket file name */
+      GString *socket_filename;
+      if (!strcmp(SpeechdOptions.socket_name, "default")){
+	/* This code cannot be moved above next to conf_dir and pidpath resolution because
+	 we need to also consider the DotConf configuration, which is read in speechd_init() */
+	socket_filename = g_string_new("");
+	if (SpeechdOptions.home_speechd_dir){
+	  g_string_printf(socket_filename, "%s/speechd.sock", SpeechdOptions.home_speechd_dir);
+	}else{
+	  FATAL("Socket name file not set and user has no home directory");
+	}
+      }else{
+	socket_filename = g_string_new(SpeechdOptions.socket_name);
+      }
+      MSG(2, "Speech Dispatcher will use local unix socket: %s", socket_filename->str);
 
-    /* Initialize socket functionality */
-    server_socket = socket(AF_INET, SOCK_STREAM, 0);
-    {
-      const int flag = 1;
-      if (setsockopt(server_socket, SOL_SOCKET, SO_REUSEADDR, &flag,
-                     sizeof (int)))
-        MSG(2,"Error: Setting socket option failed!");
+      /* Delete an old socket file if it exists */
+      if (g_file_test(socket_filename->str, G_FILE_TEST_EXISTS))
+	  if (g_unlink(socket_filename->str) == -1)
+	    FATAL("Local socket file exists but impossible to delete. Wrong permissions?");
+      
+      /* Connect and start listening on local unix socket */
+      server_socket = make_local_socket(socket_filename->str);
+      g_string_free(socket_filename, 1);
+    }else{
+      FATAL("Unknown communication method");
     }
 
-    server_address.sin_family = AF_INET;
-
-    /* Enable access only to localhost or for any address
-       based on LocalhostAccessOnly configuration option. */
-    if (SpeechdOptions.localhost_access_only)
-      server_address.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
-    else
-      server_address.sin_addr.s_addr = htonl(INADDR_ANY);
-
-    server_address.sin_port = htons(SpeechdOptions.port);
-
-    MSG(3,"Openning socket connection");
-    if (bind(server_socket, (struct sockaddr *)&server_address,
-             sizeof(server_address)) == -1){
-        MSG(1, "bind() failed: %s", strerror(errno));
-        FATAL("Couldn't open socket, try a few minutes later.");
-    }
-
-    /* Create a connection queue and initialize readfds to handle input
-       from server_socket. */
-    if (listen(server_socket, 50) == -1)
-        FATAL("listen() failed, another Speech Dispatcher running?");
-
     /* Fork, set uid, chdir, etc. */
     if (spd_mode == SPD_MODE_DAEMON){
-        daemon(0,0);	   
+        daemon(0,0);
         /* Re-create the pid file under this process */
         unlink(SpeechdOptions.pid_file);
         if (create_pid_file() == -1) return -1;
@@ -888,19 +1007,17 @@ main(int argc, char *argv[])
     SpeechdStatus.max_fd = server_socket;
 
     /* Now wait for clients and requests. */   
-    MSG(1, "Speech Dispatcher "VERSION" started on port %d and waiting for clients ...",
-	SpeechdOptions.port);
+    MSG(1, "Speech Dispatcher started and waiting for clients ...");
     while (1) {
         testfds = readfds;
 
         if (select(FD_SETSIZE, &testfds, (fd_set *)0, (fd_set *)0, NULL) >= 1){
             /* Once we know we've got activity,
              * we find which descriptor it's on by checking each in turn using FD_ISSET. */
-
             for (fd = 0; fd <= SpeechdStatus.max_fd && fd < FD_SETSIZE; fd++) {
                 if (FD_ISSET(fd,&testfds)){
                     MSG(4,"Activity on fd %d ...",fd);
-				
+		    
                     if (fd == server_socket){ 
                         /* server activity (new client) */
                         ret = speechd_connection_new(server_socket);
