@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: speechd.h,v 1.63 2008-02-08 10:01:09 hanke Exp $
+ * $Id: speechd.h,v 1.64 2008-06-27 12:29:07 hanke Exp $
  */
 
 #ifndef SPEECHDH
@@ -53,10 +53,13 @@
 #include <sys/sem.h>
 
 /* Definition of semun needed for semaphore manipulation */
-#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
-/* union semun is defined by including <sys/sem.h> */
+/* TODO: This fixes compilation for Mac OS X but might not be a correct
+   solution for other platforms. A better check is needed, possibly including
+   _POSIX_C_SOURCE and friends*/
+#if (defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)) || defined(__APPLE__)
+ /* union semun is defined by including <sys/sem.h> */
 #else
-/* according to X/OPEN we have to define it ourselves */
+ /* according to X/OPEN we have to define it ourselves */
 union semun {
     int val;                    /* value for SETVAL */
     struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
@@ -106,6 +109,10 @@ typedef struct{
 #include "speaking.h"
 
 struct{
+    char *communication_method;
+    int communication_method_set;
+    char *socket_name;
+    int socket_name_set;
     int port, port_set;
     int localhost_access_only, localhost_access_only_set;
     int log_level, log_level_set;
@@ -114,6 +121,10 @@ struct{
     char *conf_dir;
     char *home_speechd_dir;
     char *log_dir;
+    int spawn;
+    int debug;
+    char *debug_destination;
+    char *debug_logfile;
     int max_history_messages;	/* Maximum of messages in history before they expire */
 }SpeechdOptions;
 
@@ -189,6 +200,7 @@ void MSG2(int level, char* kind, char *format, ...);
 FILE *logfile;
 FILE *custom_logfile;
 char *custom_log_kind;
+FILE *debug_logfile;
 
 /* For debugging purposes, does nothing */
 void fatal_error(void);
@@ -209,6 +221,9 @@ int speechd_connection_destroy(int fd);
 gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user);
 gboolean speechd_modules_terminate(gpointer key, gpointer value, gpointer user);
 void speechd_modules_reload(gpointer key, gpointer value, gpointer user);
+void speechd_modules_debug(void);
+void speechd_modules_nodebug(void);
+
 void speechd_reload_dead_modules(int sig);
 void speechd_options_init(void);
 void speechd_init(void);
