@@ -25,12 +25,12 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#include <sys/un.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#include <ctype.h>
 
 #include "def.h"
 
@@ -38,6 +38,22 @@
 
 int sockk;
 
+#ifdef __SUNPRO_C
+/* Added by Willie Walker - strcasestr is a gcc-ism
+ */
+char *strcasestr (const char *a, const char *b)
+{
+       size_t l;
+       char f[3];
+
+       snprintf (f, sizeof(f), "%c%c", tolower(*b), toupper(*b));
+       for (l = strcspn(a, f); l != strlen(a); l += strcspn(a+l+1, f) + 1)
+               if (strncasecmp(a+l, b, strlen(b)) == 0)
+                       return (a + l);
+       return NULL;
+}
+#endif /* __SUNPRO_C */
+
 char*
 send_data(int fd, char *message, int wfr)
 {
@@ -84,30 +100,84 @@ wait_for(int fd, char* event)
     fflush(NULL);
 }
 
-int 
-init(char* client_name, char* conn_name)
+/*
+ * set_socket_path: establish the pathname that our Unix socket should
+ * have.  If the SPEECHD_SOCKET environment variable is set, then that
+ * will be our pathname.  Otherwise, the pathname
+ * is ~/.speech-dispatcher/speechd.sock.
+ */
+
+void
+set_socket_path(struct sockaddr_un *address)
 {
-  int sockfd;
-  struct sockaddr_in address;
-  char *env_port;
-  int port;
+	const char default_socket_path[] = ".speech-dispatcher/speechd.sock";
+	size_t default_path_length = strlen(default_socket_path);
+	size_t path_max = sizeof(address->sun_path);
+	char *path;
+	char *pathcopy = NULL;
+	char *home_dir;
+	size_t home_dir_length;
 
-  env_port = getenv("SPEECHD_PORT");
-  if (env_port != NULL)
-      port = strtol(env_port, NULL, 10);
-  else
-      port = SPEECHD_DEFAULT_PORT;
+	path = getenv("SPEECHD_SOCKET");
+	if ((path == NULL) || (strlen(path) == 0)) {
+		home_dir = getenv("HOME");
+		if (home_dir == NULL)
+			FATAL
+			    ("Unable to find your home directory.  Cannot run tests.");
 
-  address.sin_addr.s_addr = inet_addr("127.0.0.1");
-  address.sin_port = htons(port);
-  address.sin_family = AF_INET;
-  sockfd = socket(AF_INET, SOCK_STREAM, 0);
+		home_dir_length = strlen(home_dir);
+		if (home_dir_length == 0)
+			FATAL
+			    ("Unable to find your home directory.  Cannot run tests.");
 
-  /* connect to server */
-  if (connect(sockfd, (struct sockaddr *)&address, sizeof(address)) == -1)
-      return 0;
+		pathcopy = malloc(default_path_length + home_dir_length + 2);
+		/* The + 2 accounts for an extra slash. */
+		if (pathcopy == NULL)
+			FATAL("Out of memory!");
 
-  return sockfd;
+		strcpy(pathcopy, home_dir);
+		if (pathcopy[home_dir_length - 1] != '/') {
+			pathcopy[home_dir_length] = '/';
+			pathcopy[home_dir_length + 1] = '\0';
+		}
+		strcat(pathcopy, default_socket_path);
+		path = pathcopy;
+	}
+
+	strncpy(address->sun_path, path, path_max - 1);
+	address->sun_path[path_max - 1] = '\0';
+
+	if (pathcopy)
+		free(pathcopy);	/* Don't need it anymore. */
+}
+
+/*
+ * init: create and connect our Unix-domain socket.
+ * Returns the file descriptor of the socket on success, or -1 on
+ * failure.
+ */
+
+int
+init(void)
+{
+	int sockfd;
+	int connect_success;
+	struct sockaddr_un address;
+
+	set_socket_path(&address);
+	address.sun_family = AF_UNIX;
+	sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (sockfd != -1) {
+		connect_success =
+		    connect(sockfd, (struct sockaddr *)&address,
+			    SUN_LEN(&address));
+		if (connect_success == -1) {
+			close(sockfd);
+			sockfd = -1;
+		}
+	}
+
+	return sockfd;
 }
 
 int
@@ -148,8 +218,8 @@ main(int argc, char* argv[])
     line = malloc(1024 * sizeof(char));
     reply = malloc(4096 * sizeof(char));
 
-    sockk = init("run_test","user_test");
-    if(sockk == 0) FATAL("Can't connect to Speech Dispatcher");
+    sockk = init();
+    if(sockk == -1) FATAL("Can't connect to Speech Dispatcher");
 
     assert(line != 0);
 
